import { deepMerge } from './chunk-KNPBNSJ7.js';
import { MastraBase } from './chunk-WLG2YHOZ.js';
import { __name, __publicField } from './chunk-AJJZUHB4.js';

// src/memory/index.ts
var _MastraMemory = class _MastraMemory extends MastraBase {
  constructor(config) {
    super({
      component: "MEMORY",
      name: config.name
    });
    __publicField(this, "MAX_CONTEXT_TOKENS");
    __publicField(this, "storage");
    __publicField(this, "vector");
    __publicField(this, "embedding");
    __publicField(this, "threadConfig", {
      lastMessages: 40,
      semanticRecall: false
    });
    this.storage = config.storage;
    if (config.vector) {
      this.vector = config.vector;
      this.threadConfig.semanticRecall = true;
    }
    if (`embedding` in config) {
      this.embedding = config.embedding;
    }
    if (config.options) {
      this.threadConfig = this.getMergedThreadConfig(config.options);
    }
  }
  /**
  * Get a system message to inject into the conversation.
  * This will be called before each conversation turn.
  * Implementations can override this to inject custom system messages.
  */
  async getSystemMessage(_input) {
    return null;
  }
  parseEmbeddingOptions() {
    if (!this.embedding) {
      throw new Error(`Cannot use vector features without setting new Memory({ embedding: { ... } })`);
    }
    return this.embedding;
  }
  getMergedThreadConfig(config) {
    return deepMerge(this.threadConfig, config || {});
  }
  estimateTokens(text) {
    return Math.ceil(text.split(" ").length * 1.3);
  }
  parseMessages(messages) {
    return messages.map((msg) => ({
      ...msg,
      content: typeof msg.content === "string" && (msg.content.startsWith("[") || msg.content.startsWith("{")) ? JSON.parse(msg.content) : msg.content
    }));
  }
  convertToUIMessages(messages) {
    function addToolMessageToChat({ toolMessage, messages: messages2, toolResultContents }) {
      const chatMessages2 = messages2.map((message) => {
        if (message.toolInvocations) {
          return {
            ...message,
            toolInvocations: message.toolInvocations.map((toolInvocation) => {
              const toolResult = toolMessage.content.find((tool) => tool.toolCallId === toolInvocation.toolCallId);
              if (toolResult) {
                return {
                  ...toolInvocation,
                  state: "result",
                  result: toolResult.result
                };
              }
              return toolInvocation;
            })
          };
        }
        return message;
      });
      const resultContents = [
        ...toolResultContents,
        ...toolMessage.content
      ];
      return {
        chatMessages: chatMessages2,
        toolResultContents: resultContents
      };
    }
    __name(addToolMessageToChat, "addToolMessageToChat");
    const { chatMessages } = messages.reduce((obj, message) => {
      if (message.role === "tool") {
        return addToolMessageToChat({
          toolMessage: message,
          messages: obj.chatMessages,
          toolResultContents: obj.toolResultContents
        });
      }
      let textContent = "";
      let toolInvocations = [];
      if (typeof message.content === "string") {
        textContent = message.content;
      } else if (Array.isArray(message.content)) {
        for (const content of message.content) {
          if (content.type === "text") {
            textContent += content.text;
          } else if (content.type === "tool-call") {
            const toolResult = obj.toolResultContents.find((tool) => tool.toolCallId === content.toolCallId);
            toolInvocations.push({
              state: toolResult ? "result" : "call",
              toolCallId: content.toolCallId,
              toolName: content.toolName,
              args: content.args,
              result: toolResult?.result
            });
          }
        }
      }
      obj.chatMessages.push({
        id: message.id,
        role: message.role,
        content: textContent,
        toolInvocations
      });
      return obj;
    }, {
      chatMessages: [],
      toolResultContents: []
    });
    return chatMessages;
  }
  /**
  * Helper method to create a new thread
  * @param title - Optional title for the thread
  * @param metadata - Optional metadata for the thread
  * @returns Promise resolving to the created thread
  */
  async createThread({ threadId, resourceId, title, metadata, memoryConfig }) {
    const thread = {
      id: threadId || this.generateId(),
      title,
      resourceId,
      createdAt: /* @__PURE__ */ new Date(),
      updatedAt: /* @__PURE__ */ new Date(),
      metadata
    };
    return this.saveThread({
      thread,
      memoryConfig
    });
  }
  /**
  * Helper method to add a single message to a thread
  * @param threadId - The thread to add the message to
  * @param content - The message content
  * @param role - The role of the message sender
  * @param type - The type of the message
  * @param toolNames - Optional array of tool names that were called
  * @param toolCallArgs - Optional array of tool call arguments
  * @param toolCallIds - Optional array of tool call ids
  * @returns Promise resolving to the saved message
  */
  async addMessage({ threadId, config, content, role, type, toolNames, toolCallArgs, toolCallIds }) {
    const message = {
      id: this.generateId(),
      content,
      role,
      createdAt: /* @__PURE__ */ new Date(),
      threadId,
      type,
      toolNames,
      toolCallArgs,
      toolCallIds
    };
    const savedMessages = await this.saveMessages({
      messages: [
        message
      ],
      memoryConfig: config
    });
    return savedMessages[0];
  }
  /**
  * Generates a unique identifier
  * @returns A unique string ID
  */
  generateId() {
    return crypto.randomUUID();
  }
};
__name(_MastraMemory, "MastraMemory");
var MastraMemory = _MastraMemory;

export { MastraMemory };
