import { MastraBase } from './chunk-WLG2YHOZ.js';
import { __name, __publicField } from './chunk-AJJZUHB4.js';

// src/engine/types.ts
var PropertyType = /* @__PURE__ */ function(PropertyType2) {
  PropertyType2["LONG_TEXT"] = "LONG_TEXT";
  PropertyType2["SINGLE_LINE_TEXT"] = "SINGLE_LINE_TEXT";
  PropertyType2["SINGLE_SELECT"] = "SINGLE_SELECT";
  PropertyType2["MULTI_SELECT"] = "MULTI_SELECT";
  PropertyType2["CHECKBOX"] = "CHECKBOX";
  PropertyType2["DATE"] = "DATE";
  PropertyType2["USER"] = "USER";
  PropertyType2["BADGE_LIST"] = "BADGE_LIST";
  PropertyType2["CURRENCY"] = "CURRENCY";
  PropertyType2["URL"] = "URL";
  PropertyType2["PHONE"] = "PHONE";
  PropertyType2["CONTACT"] = "CONTACT";
  PropertyType2["COMPANY"] = "COMPANY";
  PropertyType2["PERSON"] = "PERSON";
  PropertyType2["ENRICHMENT"] = "ENRICHMENT";
  PropertyType2["COMPOSITE"] = "COMPOSITE";
  PropertyType2["BOOLEAN"] = "BOOLEAN";
  PropertyType2["NUMBER"] = "NUMBER";
  PropertyType2["FLOAT"] = "FLOAT";
  PropertyType2["JSON_OBJECT"] = "JSON_OBJECT";
  PropertyType2["JSON_ARRAY"] = "JSON_ARRAY";
  return PropertyType2;
}({});
var FilterOperators = /* @__PURE__ */ function(FilterOperators2) {
  FilterOperators2["IS"] = "is";
  FilterOperators2["EQUAL"] = "eq";
  FilterOperators2["NOT_EQUAL"] = "not_eq";
  FilterOperators2["CONTAINS"] = "contains";
  FilterOperators2["IN"] = "in";
  FilterOperators2["NOT_IN"] = "not_in";
  FilterOperators2["GREATER_THAN"] = "gt";
  FilterOperators2["LESS_THAN"] = "lt";
  FilterOperators2["NOT_CONTAINS"] = "not_contains";
  FilterOperators2["GREATER_THAN_OR_EQUAL"] = "gte";
  FilterOperators2["LESS_THAN_OR_EQUAL"] = "lte";
  FilterOperators2["OP"] = "op";
  FilterOperators2["SET"] = "set";
  FilterOperators2["NOT_SET"] = "not_set";
  return FilterOperators2;
}({});

// src/engine/engine.ts
var _MastraEngine = class _MastraEngine extends MastraBase {
  /**
  * Initializes the database connection
  * @param config Configuration object for database connection
  */
  constructor(_config) {
    super({
      component: "ENGINE",
      name: "MastraEngine"
    });
  }
};
__name(_MastraEngine, "MastraEngine");
var MastraEngine = _MastraEngine;

// src/engine/engine.mock.ts
var _MockMastraEngine = class _MockMastraEngine extends MastraEngine {
  constructor(config) {
    super(config);
    __publicField(this, "entities", []);
    __publicField(this, "records", []);
  }
  async createEntity(params) {
    const entity = {
      id: `entity_${Date.now()}`,
      name: params.name,
      connectionId: params.connectionId,
      createdAt: /* @__PURE__ */ new Date(),
      updatedAt: /* @__PURE__ */ new Date(),
      lastSyncId: null
    };
    this.entities.push(entity);
    return entity;
  }
  async getEntityById(params) {
    const entity = this.entities.find((e) => e.id === params.id);
    if (!entity) {
      throw new Error(`Entity with id ${params.id} not found`);
    }
    return entity;
  }
  async getEntity({ connectionId, name }) {
    return this.entities.find((e) => (name ? e.name === name : true) && (connectionId ? e.connectionId === connectionId : true));
  }
  async deleteEntityById({ id }) {
    const entityIndex = this.entities.findIndex((e) => e.id === id);
    if (entityIndex === -1) {
      throw new Error(`Entity with id ${id} not found`);
    }
    const [deletedEntity] = this.entities.splice(entityIndex, 1);
    this.records = this.records.filter((r) => r.entityId !== id);
    return deletedEntity;
  }
  async upsertRecords(params) {
    await this.getEntityById({
      id: params.entityId
    });
    for (const record of params.records) {
      const existingRecordIndex = this.records.findIndex((r) => r.entityId === params.entityId && r.externalId === record.externalId);
      const fullRecord = {
        id: `record_${Date.now()}_${Math.random()}`,
        entityId: params.entityId,
        externalId: record.externalId,
        data: record.data,
        entityType: record.entityType,
        createdAt: /* @__PURE__ */ new Date(),
        updatedAt: /* @__PURE__ */ new Date()
      };
      if (existingRecordIndex !== -1) {
        this.records[existingRecordIndex] = fullRecord;
      } else {
        this.records.push(fullRecord);
      }
    }
  }
  async getRecordsByEntityId(params) {
    return this.records.filter((r) => r.entityId === params.entityId);
  }
  async getRecordsByEntityNameAndExternalId({ entityName, connectionId, externalId }) {
    const entity = await this.getEntity({
      name: entityName,
      connectionId
    });
    if (!entity) {
      return [];
    }
    return this.records.filter((r) => r.entityId === entity.id && r.externalId === externalId);
  }
  async getRecordsByEntityName({ name, connectionId }) {
    const entity = await this.getEntity({
      name,
      connectionId
    });
    if (!entity) {
      return [];
    }
    return this.getRecordsByEntityId({
      entityId: entity.id
    });
  }
  async getRecords({ entityName, connectionId, options }) {
    const entity = await this.getEntity({
      name: entityName,
      connectionId
    });
    if (!entity) {
      return [];
    }
    let records = await this.getRecordsByEntityId({
      entityId: entity.id
    });
    if (options.filters) {
      records = records.filter(() => {
        return true;
      });
    }
    if (options.sort) {
      records = records.sort(() => {
        return 0;
      });
    }
    if (options.limit) {
      const start = options.offset || 0;
      records = records.slice(start, start + options.limit);
    }
    return records;
  }
  async syncRecords({ connectionId, name, records, lastSyncId }) {
    let entity = await this.getEntity({
      name,
      connectionId
    });
    if (!entity) {
      entity = await this.createEntity({
        name,
        connectionId
      });
    }
    await this.upsertRecords({
      entityId: entity.id,
      records: records.map((record) => ({
        ...record,
        lastSyncId,
        entityType: name
      }))
    });
  }
};
__name(_MockMastraEngine, "MockMastraEngine");
var MockMastraEngine = _MockMastraEngine;

export { FilterOperators, MastraEngine, MockMastraEngine, PropertyType };
