import { executeHook, AvailableHooks } from './chunk-HBTQNIAX.js';
import { LLM } from './chunk-Y7YCQUOD.js';
import { InstrumentClass } from './chunk-6ZVFVYLE.js';
import { MastraBase } from './chunk-WLG2YHOZ.js';
import { RegisteredLogger, LogLevel } from './chunk-IO3KMCXV.js';
import { __name, __publicField, __privateAdd, __privateSet, __privateGet } from './chunk-AJJZUHB4.js';
import { randomUUID } from 'crypto';
import { z } from 'zod';
import 'dotenv/config';

function _ts_decorate(decorators, target, key, desc) {
  var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
  if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
  else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
  return c > 3 && r && Object.defineProperty(target, key, r), r;
}
__name(_ts_decorate, "_ts_decorate");
function _ts_metadata(k, v) {
  if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
}
__name(_ts_metadata, "_ts_metadata");
var _mastra, _memory;
var _Agent = class _Agent extends MastraBase {
  constructor(config) {
    super({
      component: RegisteredLogger.AGENT
    });
    __publicField(this, "name");
    // @ts-ignore
    __publicField(this, "llm");
    __publicField(this, "instructions");
    __publicField(this, "model");
    __privateAdd(this, _mastra);
    __privateAdd(this, _memory);
    __publicField(this, "tools");
    __publicField(this, "metrics");
    this.name = config.name;
    this.instructions = config.instructions;
    if (!config.model && !config.llm) {
      throw new Error("Either model or llm is required");
    }
    if (config.llm) {
      this.llm = config.llm;
    } else if (config.model) {
      this.model = config.model;
      this.llm = new LLM({
        model: config.model
      });
    }
    this.tools = {};
    this.metrics = {};
    if (config.tools) {
      this.tools = config.tools;
    }
    if (config.mastra) {
      __privateSet(this, _mastra, config.mastra);
    }
    if (config.metrics) {
      this.metrics = config.metrics;
    }
    if (config.memory) {
      __privateSet(this, _memory, config.memory);
    }
  }
  hasOwnMemory() {
    return Boolean(__privateGet(this, _memory));
  }
  getMemory() {
    return __privateGet(this, _memory) ?? __privateGet(this, _mastra)?.memory;
  }
  __registerPrimitives(p) {
    if (p.telemetry) {
      this.__setTelemetry(p.telemetry);
    }
    if (p.logger) {
      this.__setLogger(p.logger);
    }
    this.llm.__registerPrimitives(p);
    __privateSet(this, _mastra, p);
    this.logger.debug(`[Agents:${this.name}] initialized.`, {
      model: this.model,
      name: this.name
    });
  }
  /**
  * Set the concrete tools for the agent
  * @param tools
  */
  __setTools(tools) {
    this.tools = tools;
    this.logger.debug(`[Agents:${this.name}] Tools set for agent ${this.name}`, {
      model: this.model,
      name: this.name
    });
  }
  async generateTitleFromUserMessage({ message }) {
    const { object } = await this.llm.__textObject({
      messages: [
        {
          role: "system",
          content: `

      - you will generate a short title based on the first message a user begins a conversation with
      - ensure it is not more than 80 characters long
      - the title should be a summary of the user's message
      - do not use quotes or colons`
        },
        {
          role: "user",
          content: JSON.stringify(message)
        }
      ],
      structuredOutput: z.object({
        title: z.string()
      })
    });
    return object.title;
  }
  getMostRecentUserMessage(messages) {
    const userMessages = messages.filter((message) => message.role === "user");
    return userMessages.at(-1);
  }
  async genTitle(userMessage) {
    let title = "New Thread";
    try {
      if (userMessage) {
        title = await this.generateTitleFromUserMessage({
          message: userMessage
        });
      }
    } catch (e) {
      console.error("Error generating title:", e);
    }
    return title;
  }
  async saveMemory({ threadId, memoryConfig, resourceId, userMessages, runId }) {
    const userMessage = this.getMostRecentUserMessage(userMessages);
    const memory = this.getMemory();
    if (memory) {
      let thread;
      if (!threadId) {
        this.logger.debug(`No threadId, creating new thread for agent ${this.name}`, {
          runId: runId || this.name
        });
        const title = await this.genTitle(userMessage);
        thread = await memory.createThread({
          threadId,
          resourceId,
          title,
          memoryConfig
        });
      } else {
        thread = await memory.getThreadById({
          threadId
        });
        if (!thread) {
          this.logger.debug(`Thread not found, creating new thread for agent ${this.name}`, {
            runId: runId || this.name
          });
          const title = await this.genTitle(userMessage);
          thread = await memory.createThread({
            threadId,
            resourceId,
            title,
            memoryConfig
          });
        }
      }
      const newMessages = userMessage ? [
        userMessage
      ] : userMessages;
      if (thread) {
        const messages = newMessages.map((u) => {
          return {
            id: this.getMemory()?.generateId(),
            createdAt: /* @__PURE__ */ new Date(),
            threadId: thread.id,
            ...u,
            content: u.content,
            role: u.role,
            type: "text"
          };
        });
        const contextCallMessages = [
          {
            role: "system",
            content: `

             Analyze this message to determine if the user is referring to a previous conversation with the LLM.
             Specifically, identify if the user wants to reference specific information from that chat or if they want the LLM to use the previous chat messages as context for the current conversation.
             Extract any date ranges mentioned in the user message that could help identify the previous chat.
             Return dates in ISO format.
             If no specific dates are mentioned but time periods are (like "last week" or "past month"), calculate the appropriate date range.
             For the end date, return the date 1 day after the end of the time period.
             Today's date is ${(/* @__PURE__ */ new Date()).toISOString()}`
          },
          ...newMessages
        ];
        let context;
        try {
          context = await this.llm.__textObject({
            messages: contextCallMessages,
            structuredOutput: z.object({
              usesContext: z.boolean(),
              startDate: z.date(),
              endDate: z.date()
            })
          });
          this.logger.debug("Text Object result", {
            contextObject: JSON.stringify(context.object, null, 2),
            runId: runId || this.name
          });
        } catch (e) {
          if (e instanceof Error) {
            this.log(LogLevel.DEBUG, `No context found: ${e.message}`);
          }
        }
        const memoryMessages = threadId && memory ? (await memory.rememberMessages({
          threadId,
          config: memoryConfig,
          vectorMessageSearch: messages.slice(-1).map((m) => {
            if (typeof m === `string`) {
              return m;
            }
            return m?.content || ``;
          }).join(`
`)
        })).messages : [];
        if (memory) {
          await memory.saveMessages({
            messages,
            memoryConfig
          });
        }
        this.log(LogLevel.DEBUG, "Saved messages to memory", {
          threadId: thread.id,
          runId
        });
        const memorySystemMessage = memory && threadId ? await memory.getSystemMessage({
          threadId,
          memoryConfig
        }) : null;
        return {
          threadId: thread.id,
          messages: [
            {
              role: "system",
              content: `

             Analyze this message to determine if the user is referring to a previous conversation with the LLM.
             Specifically, identify if the user wants to reference specific information from that chat or if they want the LLM to use the previous chat messages as context for the current conversation.
             Extract any date ranges mentioned in the user message that could help identify the previous chat.
             Return dates in ISO format.
             If no specific dates are mentioned but time periods are (like "last week" or "past month"), calculate the appropriate date range.
             For the end date, return the date 1 day after the end of the time period.
             Today's date is ${(/* @__PURE__ */ new Date()).toISOString()} and the time is ${(/* @__PURE__ */ new Date()).toLocaleTimeString("en-US", {
                hour: "numeric",
                minute: "numeric",
                hour12: true
              })} ${memorySystemMessage ? `

${memorySystemMessage}` : ""}`
            },
            ...memoryMessages,
            ...newMessages
          ]
        };
      }
      return {
        threadId: thread?.id || threadId || "",
        messages: userMessages
      };
    }
    return {
      threadId: threadId || "",
      messages: userMessages
    };
  }
  async saveResponse({ result, threadId, resourceId, runId, memoryConfig }) {
    const { response } = result;
    try {
      if (response.messages) {
        const ms = Array.isArray(response.messages) ? response.messages : [
          response.messages
        ];
        const responseMessagesWithoutIncompleteToolCalls = this.sanitizeResponseMessages(ms);
        const memory = this.getMemory();
        if (memory) {
          this.logger.debug(`[Agent:${this.name}] - Memory persistence: store=${__privateGet(this, _mastra)?.memory?.constructor.name} threadId=${threadId}`, {
            runId,
            resourceId,
            threadId,
            memoryStore: __privateGet(this, _mastra)?.memory?.constructor.name
          });
          await memory.saveMessages({
            memoryConfig,
            messages: responseMessagesWithoutIncompleteToolCalls.map((message) => {
              const messageId = randomUUID();
              let toolCallIds;
              let toolCallArgs;
              let toolNames;
              let type = "text";
              if (message.role === "tool") {
                toolCallIds = message.content.map((content) => content.toolCallId);
                type = "tool-result";
              }
              if (message.role === "assistant") {
                const assistantContent = message.content;
                const assistantToolCalls = assistantContent.map((content) => {
                  if (content.type === "tool-call") {
                    return {
                      toolCallId: content.toolCallId,
                      toolArgs: content.args,
                      toolName: content.toolName
                    };
                  }
                  return void 0;
                })?.filter(Boolean);
                toolCallIds = assistantToolCalls?.map((toolCall) => toolCall.toolCallId);
                toolCallArgs = assistantToolCalls?.map((toolCall) => toolCall.toolArgs);
                toolNames = assistantToolCalls?.map((toolCall) => toolCall.toolName);
                type = assistantContent?.[0]?.type;
              }
              return {
                id: messageId,
                threadId,
                role: message.role,
                content: message.content,
                createdAt: /* @__PURE__ */ new Date(),
                toolCallIds: toolCallIds?.length ? toolCallIds : void 0,
                toolCallArgs: toolCallArgs?.length ? toolCallArgs : void 0,
                toolNames: toolNames?.length ? toolNames : void 0,
                type
              };
            })
          });
        }
      }
    } catch (err) {
      this.logger.error(`[Agent:${this.name}] - Failed to save assistant response`, {
        error: err,
        runId
      });
    }
  }
  sanitizeResponseMessages(messages) {
    let toolResultIds = [];
    for (const message of messages) {
      if (message.role === "tool") {
        for (const content of message.content) {
          if (content.type === "tool-result") {
            toolResultIds.push(content.toolCallId);
          }
        }
      }
    }
    const messagesBySanitizedContent = messages.map((message) => {
      if (message.role !== "assistant") return message;
      if (typeof message.content === "string") return message;
      const sanitizedContent = message.content.filter((content) => content.type === "tool-call" ? toolResultIds.includes(content.toolCallId) : content.type === "text" ? content.text.length > 0 : true);
      return {
        ...message,
        content: sanitizedContent
      };
    });
    return messagesBySanitizedContent.filter((message) => message.content.length > 0);
  }
  convertTools({
    toolsets,
    // threadId,
    runId
  }) {
    this.logger.debug(`[Agents:${this.name}] - Assigning tools`, {
      runId
    });
    const converted = Object.entries(this.tools || {}).reduce((memo, value) => {
      const k = value[0];
      const tool = this.tools[k];
      if (tool) {
        memo[k] = {
          description: tool.description,
          parameters: tool.inputSchema,
          execute: /* @__PURE__ */ __name(async (args) => {
            try {
              this.logger.debug(`[Agent:${this.name}] - Executing tool ${k}`, {
                name: k,
                description: tool.description,
                args
              });
              return tool.execute({
                context: args,
                mastra: __privateGet(this, _mastra)
              });
            } catch (err) {
              this.logger.error(`[Agent:${this.name}] - Failed execution`, {
                error: err,
                runId
              });
              throw err;
            }
          }, "execute")
        };
      }
      return memo;
    }, {});
    const toolsFromToolsetsConverted = {
      ...converted
    };
    const toolsFromToolsets = Object.values(toolsets || {});
    if (toolsFromToolsets.length > 0) {
      this.logger.debug(`[Agent:${this.name}] - Adding tools from toolsets ${Object.keys(toolsets || {}).join(", ")}`, {
        runId
      });
      toolsFromToolsets.forEach((toolset) => {
        Object.entries(toolset).forEach(([toolName, tool]) => {
          const toolObj = tool;
          toolsFromToolsetsConverted[toolName] = {
            description: toolObj.description || "",
            parameters: toolObj.inputSchema,
            execute: /* @__PURE__ */ __name(async (args) => {
              try {
                this.logger.debug(`[Agent:${this.name}] - Executing tool ${toolName}`, {
                  name: toolName,
                  description: toolObj.description,
                  args
                });
                return toolObj.execute({
                  context: args
                });
              } catch (err) {
                this.logger.error(`[Agent:${this.name}] - Failed toolset execution`, {
                  error: err,
                  runId
                });
                throw err;
              }
            }, "execute")
          };
        });
      });
    }
    return toolsFromToolsetsConverted;
  }
  async preExecute({ resourceId, runId, threadId, memoryConfig, messages }) {
    let coreMessages = [];
    let threadIdToUse = threadId;
    this.log(LogLevel.INFO, `Saving user messages in memory for agent ${this.name}`, {
      runId
    });
    const saveMessageResponse = await this.saveMemory({
      threadId,
      resourceId,
      userMessages: messages,
      memoryConfig
    });
    coreMessages = saveMessageResponse.messages;
    threadIdToUse = saveMessageResponse.threadId;
    return {
      coreMessages,
      threadIdToUse
    };
  }
  __primitive({ messages, context, threadId, memoryConfig, resourceId, runId, toolsets }) {
    return {
      before: /* @__PURE__ */ __name(async () => {
        if (process.env.NODE_ENV !== "test") {
          this.logger.debug(`[Agents:${this.name}] - Starting generation`, {
            runId
          });
        }
        const systemMessage = {
          role: "system",
          content: `${this.instructions}. Today's date is ${(/* @__PURE__ */ new Date()).toISOString()}`
        };
        let coreMessages = messages;
        let threadIdToUse = threadId;
        if (this.getMemory() && resourceId) {
          this.logger.debug(`[Agent:${this.name}] - Memory persistence enabled: store=${__privateGet(this, _mastra)?.memory?.constructor.name}, resourceId=${resourceId}`, {
            runId,
            resourceId,
            threadId: threadIdToUse,
            memoryStore: __privateGet(this, _mastra)?.memory?.constructor.name
          });
          const preExecuteResult = await this.preExecute({
            resourceId,
            runId,
            threadId: threadIdToUse,
            memoryConfig,
            messages
          });
          coreMessages = preExecuteResult.coreMessages;
          threadIdToUse = preExecuteResult.threadIdToUse;
        }
        let convertedTools;
        if (toolsets && Object.keys(toolsets || {}).length > 0 || this.getMemory() && resourceId || __privateGet(this, _mastra)?.engine) {
          const reasons = [];
          if (toolsets && Object.keys(toolsets || {}).length > 0) {
            reasons.push(`toolsets present (${Object.keys(toolsets || {}).length} tools)`);
          }
          if (this.getMemory() && resourceId) {
            reasons.push("memory and resourceId available");
          }
          if (__privateGet(this, _mastra)?.engine) {
            reasons.push("mastra engine enabled");
          }
          this.logger.debug(`[Agent:${this.name}] - Enhancing tools: ${reasons.join(", ")}`, {
            runId,
            toolsets: toolsets ? Object.keys(toolsets) : undefined,
            hasMemory: !!this.getMemory(),
            hasResourceId: !!resourceId,
            hasEngine: !!__privateGet(this, _mastra)?.engine
          });
          convertedTools = this.convertTools({
            toolsets,
            threadId: threadIdToUse,
            runId
          });
        }
        const messageObjects = [
          systemMessage,
          ...context || [],
          ...coreMessages
        ];
        return {
          messageObjects,
          convertedTools,
          threadId: threadIdToUse
        };
      }, "before"),
      after: /* @__PURE__ */ __name(async ({ result, threadId: threadId2, memoryConfig: memoryConfig2, outputText, runId: runId2 }) => {
        const resToLog = {
          text: result?.text,
          object: result?.object,
          toolResults: result?.toolResults,
          toolCalls: result?.toolCalls,
          usage: result?.usage,
          steps: result?.steps?.map((s) => {
            return {
              stepType: s?.stepType,
              text: result?.text,
              object: result?.object,
              toolResults: result?.toolResults,
              toolCalls: result?.toolCalls,
              usage: result?.usage
            };
          })
        };
        this.logger.debug(`[Agent:${this.name}] - Post processing LLM response`, {
          runId: runId2,
          result: resToLog,
          threadId: threadId2
        });
        if (this.getMemory() && resourceId) {
          try {
            await this.saveResponse({
              result,
              threadId: threadId2,
              resourceId,
              memoryConfig: memoryConfig2,
              runId: runId2
            });
          } catch (e) {
            this.logger.error("Error saving response", {
              error: e,
              runId: runId2,
              result: resToLog,
              threadId: threadId2
            });
          }
        }
        if (Object.keys(this.metrics || {}).length > 0) {
          const input = messages.map((message) => message.content).join("\n");
          const runIdToUse = runId2 || crypto.randomUUID();
          for (const metric of Object.values(this.metrics || {})) {
            executeHook(AvailableHooks.ON_GENERATION, {
              input,
              output: outputText,
              runId: runIdToUse,
              metric,
              agentName: this.name
            });
          }
        }
      }, "after")
    };
  }
  async generate(messages, { context, threadId: threadIdInFn, memoryOptions, resourceId, maxSteps = 5, onStepFinish, runId, toolsets, output = "text", temperature } = {}) {
    let messagesToUse = [];
    if (typeof messages === `string`) {
      messagesToUse = [
        {
          role: "user",
          content: messages
        }
      ];
    } else {
      messagesToUse = messages.map((message) => {
        if (typeof message === `string`) {
          return {
            role: "user",
            content: message
          };
        }
        return message;
      });
    }
    const runIdToUse = runId || randomUUID();
    const { before, after } = this.__primitive({
      messages: messagesToUse,
      context,
      threadId: threadIdInFn,
      memoryConfig: memoryOptions,
      resourceId,
      runId: runIdToUse,
      toolsets
    });
    const { threadId, messageObjects, convertedTools } = await before();
    if (output === "text") {
      const result2 = await this.llm.__text({
        messages: messageObjects,
        tools: this.tools,
        convertedTools,
        onStepFinish,
        maxSteps,
        runId: runIdToUse,
        temperature
      });
      const outputText2 = result2.text;
      await after({
        result: result2,
        threadId,
        memoryConfig: memoryOptions,
        outputText: outputText2,
        runId: runIdToUse
      });
      return result2;
    }
    const result = await this.llm.__textObject({
      messages: messageObjects,
      tools: this.tools,
      structuredOutput: output,
      convertedTools,
      onStepFinish,
      maxSteps,
      runId: runIdToUse,
      temperature
    });
    const outputText = JSON.stringify(result.object);
    await after({
      result,
      threadId,
      memoryConfig: memoryOptions,
      outputText,
      runId: runIdToUse
    });
    return result;
  }
  async stream(messages, { context, threadId: threadIdInFn, memoryOptions, resourceId, maxSteps = 5, onFinish, onStepFinish, runId, toolsets, output = "text", temperature } = {}) {
    const runIdToUse = runId || randomUUID();
    let messagesToUse = [];
    if (typeof messages === `string`) {
      messagesToUse = [
        {
          role: "user",
          content: messages
        }
      ];
    } else {
      messagesToUse = messages.map((message) => {
        if (typeof message === `string`) {
          return {
            role: "user",
            content: message
          };
        }
        return message;
      });
    }
    const { before, after } = this.__primitive({
      messages: messagesToUse,
      context,
      threadId: threadIdInFn,
      memoryConfig: memoryOptions,
      resourceId,
      runId: runIdToUse,
      toolsets
    });
    const { threadId, messageObjects, convertedTools } = await before();
    if (output === "text") {
      this.logger.debug(`Starting agent ${this.name} llm stream call`, {
        runId
      });
      return this.llm.__stream({
        messages: messageObjects,
        temperature,
        tools: this.tools,
        convertedTools,
        onStepFinish,
        onFinish: /* @__PURE__ */ __name(async (result) => {
          try {
            const res = JSON.parse(result) || {};
            const outputText = res.text;
            await after({
              result: res,
              threadId,
              memoryConfig: memoryOptions,
              outputText,
              runId: runIdToUse
            });
          } catch (e) {
            this.logger.error("Error saving memory on finish", {
              error: e,
              runId
            });
          }
          onFinish?.(result);
        }, "onFinish"),
        maxSteps,
        runId
      });
    }
    this.logger.debug(`Starting agent ${this.name} llm streamObject call`, {
      runId
    });
    return this.llm.__streamObject({
      messages: messageObjects,
      tools: this.tools,
      temperature,
      structuredOutput: output,
      convertedTools,
      onStepFinish,
      onFinish: /* @__PURE__ */ __name(async (result) => {
        try {
          const res = JSON.parse(result) || {};
          const outputText = JSON.stringify(res.object);
          await after({
            result: res,
            threadId,
            memoryConfig: memoryOptions,
            outputText,
            runId: runIdToUse
          });
        } catch (e) {
          this.logger.error("Error saving memory on finish", {
            error: e,
            runId
          });
        }
        onFinish?.(result);
      }, "onFinish"),
      maxSteps,
      runId
    });
  }
};
_mastra = new WeakMap();
_memory = new WeakMap();
__name(_Agent, "Agent");
var Agent = _Agent;
Agent = _ts_decorate([
  InstrumentClass({
    prefix: "agent",
    excludeMethods: [
      "__setTools",
      "__setLogger",
      "__setTelemetry",
      "log"
    ]
  }),
  _ts_metadata("design:type", Function),
  _ts_metadata("design:paramtypes", [
    typeof AgentConfig === "undefined" ? Object : AgentConfig
  ])
], Agent);

export { Agent };
