import * as ai from 'ai';
import { CoreMessage as CoreMessage$1, CoreSystemMessage as CoreSystemMessage$1, CoreAssistantMessage as CoreAssistantMessage$1, CoreUserMessage as CoreUserMessage$1, CoreToolMessage as CoreToolMessage$1, EmbedResult as EmbedResult$1, EmbedManyResult as EmbedManyResult$1, LanguageModelV1, GenerateTextResult, GenerateObjectResult, StreamTextResult, StreamObjectResult, DeepPartial, Message, UserContent, AssistantContent, ToolContent, CoreTool as CoreTool$1 } from 'ai';
import { JSONSchema7 } from 'json-schema';
import { z, ZodSchema } from 'zod';
import { MastraBase } from './base.js';
import { M as Metric } from './metric-D2V4CR8D.js';
import { T as Telemetry } from './telemetry-oCUM52DG.js';
import { B as BaseLogMessage, R as RegisteredLogger, b as Run, L as Logger } from './index-Cktbxvxx.js';
import { Query } from 'sift';
import { E as EmbeddingOptions } from './types-M16hSruO.js';
import { MastraVector } from './vector/index.js';
import { M as MastraEngine } from './engine-EwEG-4Iv.js';
import { MastraTTS } from './tts/index.js';

interface WorkflowOptions<TTriggerSchema extends z.ZodType<any> = any> {
    name: string;
    triggerSchema?: TTriggerSchema;
    retryConfig?: RetryConfig;
    mastra?: MastraPrimitives;
}
interface StepExecutionContext<TSchemaIn extends z.ZodSchema | undefined = undefined, TContext extends WorkflowContext = WorkflowContext> extends IExecutionContext<TSchemaIn, TContext> {
    runId: string;
}
interface StepAction<TId extends string, TSchemaIn extends z.ZodSchema | undefined, TSchemaOut extends z.ZodSchema | undefined, TContext extends StepExecutionContext<TSchemaIn>> extends IAction<TId, TSchemaIn, TSchemaOut, TContext> {
    retryConfig?: RetryConfig;
}
interface SimpleConditionalType {
    [key: `${string}.${string}`]: string | Query<any>;
}
type StepVariableType<TId extends string, TSchemaIn extends z.ZodSchema | undefined, TSchemaOut extends z.ZodSchema | undefined, TContext extends IExecutionContext<TSchemaIn>> = IAction<TId, TSchemaIn, TSchemaOut, TContext> | 'trigger' | {
    id: string;
};
type StepNode = {
    step: IAction<any, any, any, any>;
    config: StepDef<any, any, any, any>[any];
};
type StepGraph = {
    initial: StepNode[];
    [key: string]: StepNode[];
};
type RetryConfig = {
    attempts?: number;
    delay?: number;
};
type VariableReference<TStep extends StepVariableType<any, any, any, any>, TTriggerSchema extends z.ZodType<any>> = TStep extends IAction<any, any, any, any> ? {
    step: TStep;
    path: PathsToStringProps<ExtractSchemaType<ExtractSchemaFromStep<TStep, 'outputSchema'>>> | '' | '.';
} : TStep extends 'trigger' ? {
    step: 'trigger';
    path: PathsToStringProps<ExtractSchemaType<TTriggerSchema>> | '.' | '';
} : {
    step: {
        id: string;
    };
    path: string;
};
interface BaseCondition<TStep extends StepVariableType<any, any, any, any>, TTriggerSchema extends z.ZodType<any>> {
    ref: TStep extends IAction<any, any, any, any> ? {
        step: TStep;
        path: PathsToStringProps<ExtractSchemaType<ExtractSchemaFromStep<TStep, 'outputSchema'>>> | '' | '.' | 'status';
    } : TStep extends 'trigger' ? {
        step: 'trigger';
        path: PathsToStringProps<ExtractSchemaType<TTriggerSchema>> | '.' | '';
    } : {
        step: {
            id: string;
        };
        path: string;
    };
    query: Query<any>;
}
type ActionContext<TSchemaIn extends z.ZodType<any>> = IExecutionContext<z.infer<TSchemaIn>, WorkflowContext>;
type StepDef<TStepId extends TSteps[number]['id'], TSteps extends IAction<any, any, any, any>[], TSchemaIn extends z.ZodType<any>, TSchemaOut extends z.ZodType<any>> = Record<TStepId, {
    snapshotOnTimeout?: boolean;
    when?: Condition<any, any> | ((args: {
        context: WorkflowContext;
    }) => Promise<boolean>);
    data: TSchemaIn;
    handler: (args: ActionContext<TSchemaIn>) => Promise<z.infer<TSchemaOut>>;
}>;
type StepCondition<TStep extends StepVariableType<any, any, any, any>, TTriggerSchema extends z.ZodType<any>> = BaseCondition<TStep, TTriggerSchema> | SimpleConditionalType | {
    and: StepCondition<TStep, TTriggerSchema>[];
} | {
    or: StepCondition<TStep, TTriggerSchema>[];
};
type Condition<TStep extends StepVariableType<any, any, any, any>, TTriggerSchema extends z.ZodType<any>> = BaseCondition<TStep, TTriggerSchema> | SimpleConditionalType | {
    and: Condition<TStep, TTriggerSchema>[];
} | {
    or: Condition<TStep, TTriggerSchema>[];
};
interface StepConfig<TStep extends IAction<any, any, any, any>, CondStep extends StepVariableType<any, any, any, any>, VarStep extends StepVariableType<any, any, any, any>, TTriggerSchema extends z.ZodType<any>> {
    snapshotOnTimeout?: boolean;
    when?: Condition<CondStep, TTriggerSchema> | ((args: {
        context: WorkflowContext<TTriggerSchema>;
    }) => Promise<boolean>);
    variables?: StepInputType<TStep, 'inputSchema'> extends never ? Record<string, VariableReference<VarStep, TTriggerSchema>> : {
        [K in keyof StepInputType<TStep, 'inputSchema'>]?: VariableReference<VarStep, TTriggerSchema>;
    };
}
type StepSuccess<T> = {
    status: 'success';
    payload: T;
};
type StepSuspended = {
    status: 'suspended';
};
type StepWaiting = {
    status: 'waiting';
};
type StepFailure = {
    status: 'failed';
    error: string;
};
type StepResult<T> = StepSuccess<T> | StepFailure | StepSuspended | StepWaiting;
interface WorkflowContext<TTrigger extends z.ZodType<any> = any> {
    stepResults: Record<string, StepResult<any>>;
    triggerData: z.infer<TTrigger>;
    attempts: Record<string, number>;
    getStepPayload: <T = unknown>(stepId: string) => T | undefined;
}
interface WorkflowLogMessage extends BaseLogMessage {
    type: typeof RegisteredLogger.WORKFLOW;
    workflowName: string;
    stepId?: StepId;
    data?: unknown;
    runId?: string;
}
type WorkflowEvent = {
    type: 'RESET_TO_PENDING';
    stepId: string;
} | {
    type: 'CONDITIONS_MET';
    stepId: string;
} | {
    type: 'CONDITION_FAILED';
    stepId: string;
    error: string;
} | {
    type: 'SUSPENDED';
    stepId: string;
} | {
    type: 'WAITING';
    stepId: string;
} | {
    type: `xstate.error.actor.${string}`;
    error: Error;
} | {
    type: `xstate.done.actor.${string}`;
    output: ResolverFunctionOutput;
};
type ResolverFunctionInput = {
    stepNode: StepNode;
    context: WorkflowContext;
};
type ResolverFunctionOutput = {
    stepId: StepId;
    result: unknown;
};
type SubscriberFunctionOutput = {
    stepId: StepId;
    result: unknown;
};
type DependencyCheckOutput = {
    type: 'CONDITIONS_MET';
} | {
    type: 'CONDITION_FAILED';
    error: string;
} | {
    type: 'SUSPENDED';
} | {
    type: 'WAITING';
};
type WorkflowActors = {
    resolverFunction: {
        input: ResolverFunctionInput;
        output: ResolverFunctionOutput;
    };
    conditionCheck: {
        input: {
            context: WorkflowContext;
            stepId: string;
        };
        output: DependencyCheckOutput;
    };
    spawnSubscriberFunction: {
        input: {
            context: WorkflowContext;
            stepId: string;
        };
        output: SubscriberFunctionOutput;
    };
};
type WorkflowActionParams = {
    stepId: string;
};
type WorkflowActions = {
    type: 'updateStepResult' | 'setStepError' | 'notifyStepCompletion' | 'decrementAttemptCount';
    params: WorkflowActionParams;
};
type WorkflowState = {
    [key: string]: {
        initial: 'pending';
        states: {
            pending: {
                invoke: {
                    src: 'conditionCheck';
                    input: ({ context }: {
                        context: WorkflowContext;
                    }) => {
                        context: WorkflowContext;
                        stepId: string;
                    };
                    onDone: [
                        {
                            guard: (_: any, event: {
                                output: DependencyCheckOutput;
                            }) => boolean;
                            target: 'executing';
                        },
                        {
                            guard: (_: any, event: {
                                output: DependencyCheckOutput;
                            }) => boolean;
                            target: 'waiting';
                        }
                    ];
                };
            };
            waiting: {
                after: {
                    CHECK_INTERVAL: {
                        target: 'pending';
                    };
                };
            };
            executing: {
                invoke: {
                    src: 'resolverFunction';
                    input: ({ context }: {
                        context: WorkflowContext;
                    }) => ResolverFunctionInput;
                    onDone: {
                        target: 'completed';
                        actions: ['updateStepResult'];
                    };
                    onError: {
                        target: 'failed';
                        actions: ['setStepError'];
                    };
                };
            };
            completed: {
                type: 'final';
                entry: ['notifyStepCompletion'];
            };
            failed: {
                type: 'final';
                entry: ['notifyStepCompletion'];
            };
        };
    };
};
declare const StepIdBrand: unique symbol;
type StepId = string & {
    readonly [StepIdBrand]: typeof StepIdBrand;
};
type ExtractSchemaFromStep<TStep extends IAction<any, any, any, any>, TKey extends 'inputSchema' | 'outputSchema'> = TStep[TKey];
type ExtractStepResult<T> = T extends (data: any) => Promise<infer R> ? R : never;
type StepInputType<TStep extends IAction<any, any, any, any>, TKey extends 'inputSchema' | 'outputSchema'> = ExtractSchemaFromStep<TStep, TKey> extends infer Schema ? Schema extends z.ZodType<any> ? z.infer<Schema> : never : never;
type ExtractSchemaType<T extends z.ZodSchema> = T extends z.ZodSchema<infer V> ? V : never;
type PathsToStringProps<T> = T extends object ? {
    [K in keyof T]: T[K] extends object ? K extends string ? K | `${K}.${PathsToStringProps<T[K]>}` : never : K extends string ? K : never;
}[keyof T] : never;
interface WorkflowRunState {
    value: Record<string, string>;
    context: {
        stepResults: Record<string, {
            status: 'success' | 'failed' | 'suspended' | 'waiting';
            payload?: any;
            error?: string;
        }>;
        triggerData: Record<string, any>;
        attempts: Record<string, number>;
    };
    activePaths: Array<{
        stepPath: string[];
        stepId: string;
        status: string;
    }>;
    runId: string;
    timestamp: number;
}

type CoreTool = {
    description?: string;
    parameters: ZodSchema;
    execute: (params: any) => Promise<any>;
};
interface ToolExecutionContext<TSchemaIn extends z.ZodSchema | undefined = undefined, TContext extends WorkflowContext = WorkflowContext> extends IExecutionContext<TSchemaIn, TContext> {
}
interface ToolAction<TId extends string, TSchemaIn extends z.ZodSchema | undefined = undefined, TSchemaOut extends z.ZodSchema | undefined = undefined, TContext extends ToolExecutionContext<TSchemaIn> = ToolExecutionContext<TSchemaIn>> extends IAction<TId, TSchemaIn, TSchemaOut, TContext> {
}

type OpenAIModel = 'gpt-4' | 'gpt-4-turbo' | 'gpt-3.5-turbo' | 'gpt-4o' | 'gpt-4o-mini' | 'o1' | 'o1-mini' | 'o1-preview';
type CoreMessage = CoreMessage$1;
type CoreSystemMessage = CoreSystemMessage$1;
type CoreAssistantMessage = CoreAssistantMessage$1;
type CoreUserMessage = CoreUserMessage$1;
type CoreToolMessage = CoreToolMessage$1;
type EmbedResult<T> = EmbedResult$1<T>;
type EmbedManyResult<T> = EmbedManyResult$1<T>;
type OpenAIConfig = {
    provider: 'OPEN_AI';
    name: OpenAIModel | (string & {});
    toolChoice?: 'auto' | 'required';
    apiKey?: string;
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type GoogleModel = 'gemini-1.5-pro-latest' | 'gemini-1.5-pro' | 'gemini-1.5-flash-latest' | 'gemini-1.5-flash' | 'gemini-2.0-flash-exp-latest' | 'gemini-2.0-flash-thinking-exp-1219' | 'gemini-exp-1206';
type GoogleConfig = {
    provider: 'GOOGLE';
    name: GoogleModel | (string & {});
    toolChoice?: 'auto' | 'required';
    apiKey?: string;
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
interface GoogleGenerativeAISettings {
    /**
    Optional.
    The name of the cached content used as context to serve the prediction.
    Format: cachedContents/{cachedContent}
       */
    cachedContent?: string;
    /**
     * Optional. Enable structured output. Default is true.
     *
     * This is useful when the JSON Schema contains elements that are
     * not supported by the OpenAPI schema version that
     * Google Generative AI uses. You can use this to disable
     * structured outputs if you need to.
     */
    structuredOutputs?: boolean;
    /**
    Optional. A list of unique safety settings for blocking unsafe content.
       */
    safetySettings?: Array<{
        category: 'HARM_CATEGORY_HATE_SPEECH' | 'HARM_CATEGORY_DANGEROUS_CONTENT' | 'HARM_CATEGORY_HARASSMENT' | 'HARM_CATEGORY_SEXUALLY_EXPLICIT';
        threshold: 'HARM_BLOCK_THRESHOLD_UNSPECIFIED' | 'BLOCK_LOW_AND_ABOVE' | 'BLOCK_MEDIUM_AND_ABOVE' | 'BLOCK_ONLY_HIGH' | 'BLOCK_NONE';
    }>;
}
type AnthropicModel = 'claude-3-5-sonnet-20241022' | 'claude-3-5-sonnet-20240620' | 'claude-3-5-haiku-20241022' | 'claude-3-opus-20240229' | 'claude-3-sonnet-20240229' | 'claude-3-haiku-20240307';
type AnthropicConfig = {
    provider: 'ANTHROPIC';
    name: AnthropicModel | (string & {});
    toolChoice?: 'auto' | 'required';
    apiKey?: string;
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type GroqModel = 'llama3-groq-70b-8192-tool-use-preview' | 'llama3-groq-8b-8192-tool-use-preview' | 'gemma2-9b-it' | 'gemma-7b-it';
type GroqConfig = {
    provider: 'GROQ';
    name: GroqModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type PerplexityModel = 'llama-3.1-sonar-small-128k-online' | 'llama-3.1-sonar-large-128k-online' | 'llama-3.1-sonar-huge-128k-online' | 'llama-3.1-sonar-small-128k-chat' | 'llama-3.1-sonar-large-128k-chat' | 'llama-3.1-8b-instruct' | 'llama-3.1-70b-instruct' | 'sonar' | 'sonar-pro';
type PerplexityConfig = {
    provider: 'PERPLEXITY';
    name: PerplexityModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type TogetherAiModel = 'codellama/CodeLlama-34b-Instruct-hf' | 'upstage/SOLAR-10.7B-Instruct-v1.0' | 'mistralai/Mixtral-8x7B-v0.1' | 'WhereIsAI/UAE-Large-V1' | 'black-forest-labs/FLUX.1-depth' | 'togethercomputer/m2-bert-80M-32k-retrieval' | 'black-forest-labs/FLUX.1-canny' | 'black-forest-labs/FLUX.1-dev' | 'black-forest-labs/FLUX.1-redux' | 'BAAI/bge-large-en-v1.5' | 'meta-llama/Llama-3.2-90B-Vision-Instruct-Turbo' | 'togethercomputer/Llama-3-8b-chat-hf-int4' | 'stabilityai/stable-diffusion-xl-base-1.0' | 'Gryphe/MythoMax-L2-13b' | 'meta-llama/Meta-Llama-3-8B' | 'mistralai/Mixtral-8x7B-Instruct-v0.1' | 'deepseek-ai/deepseek-llm-67b-chat' | 'togethercomputer/m2-bert-80M-8k-retrieval' | 'llava-hf/llava-v1.6-mistral-7b-hf' | 'Qwen/Qwen2.5-Coder-32B-Instruct' | 'databricks/dbrx-instruct' | 'meta-llama/Meta-Llama-3.1-405B-Instruct-Lite-Pro' | 'meta-llama/Meta-Llama-3-8B-Instruct-Lite' | 'scb10x/scb10x-llama3-typhoon-v1-5-8b-instruct' | 'microsoft/WizardLM-2-8x22B' | 'BAAI/bge-base-en-v1.5' | 'togethercomputer/m2-bert-80M-2k-retrieval' | 'google/gemma-2b-it' | 'meta-llama/Llama-2-70b-hf' | 'mistralai/Mistral-7B-Instruct-v0.2' | 'meta-llama/LlamaGuard-2-8b' | 'mistralai/Mistral-7B-Instruct-v0.1' | 'mistralai/Mistral-7B-v0.1' | 'black-forest-labs/FLUX.1-pro' | 'black-forest-labs/FLUX.1-schnell' | 'meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo' | 'nvidia/Llama-3.1-Nemotron-70B-Instruct-HF' | 'mistralai/Mistral-7B-Instruct-v0.3' | 'Meta-Llama/Llama-Guard-7b' | 'meta-llama/Meta-Llama-3-70B-Instruct-Turbo' | 'meta-llama/Meta-Llama-3-70B-Instruct-Lite' | 'google/gemma-2-27b-it' | 'meta-llama/Llama-3-8b-chat-hf' | 'Qwen/Qwen2.5-72B-Instruct-Turbo' | 'Salesforce/Llama-Rank-V1' | 'meta-llama/Llama-Guard-3-11B-Vision-Turbo' | 'google/gemma-2-9b-it' | 'meta-llama/Llama-3.2-11B-Vision-Instruct-Turbo' | 'NousResearch/Nous-Hermes-2-Mixtral-8x7B-DPO' | 'meta-llama/Llama-3-70b-chat-hf' | 'Qwen/Qwen2.5-7B-Instruct-Turbo' | 'scb10x/scb10x-llama3-typhoon-v1-5x-4f316' | 'meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo' | 'Gryphe/MythoMax-L2-13b-Lite' | 'black-forest-labs/FLUX.1-schnell-Free' | 'meta-llama/Llama-2-7b-chat-hf' | 'meta-llama/Meta-Llama-Guard-3-8B' | 'togethercomputer/Llama-3-8b-chat-hf-int8' | 'meta-llama/Llama-Vision-Free' | 'Qwen/Qwen2-72B-Instruct' | 'mistralai/Mixtral-8x22B-Instruct-v0.1' | 'black-forest-labs/FLUX.1.1-pro' | 'meta-llama/Llama-3.2-3B-Instruct-Turbo' | 'meta-llama/Llama-2-13b-chat-hf' | 'meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo' | 'Nexusflow/NexusRaven-V2-13B' | 'bert-base-uncased' | 'WizardLM/WizardLM-13B-V1.2' | 'google/gemma-7b' | 'togethercomputer/Koala-7B' | 'zero-one-ai/Yi-34B' | 'togethercomputer/StripedHyena-Hessian-7B' | 'teknium/OpenHermes-2-Mistral-7B' | 'Qwen/Qwen2-7B-Instruct' | 'togethercomputer/guanaco-65b' | 'togethercomputer/llama-2-7b' | 'hazyresearch/M2-BERT-2k-Retrieval-Encoder-V1' | 'huggyllama/llama-7b' | 'Undi95/ReMM-SLERP-L2-13B' | 'NousResearch/Nous-Capybara-7B-V1p9' | 'lmsys/vicuna-7b-v1.3' | 'Undi95/Toppy-M-7B' | 'Qwen/Qwen2-72B' | 'NousResearch/Nous-Hermes-Llama2-70b' | 'WizardLM/WizardLM-70B-V1.0' | 'huggyllama/llama-65b' | 'lmsys/vicuna-13b-v1.5-16k' | 'openchat/openchat-3.5-1210' | 'Qwen/Qwen1.5-0.5B' | 'Qwen/Qwen1.5-4B' | 'Qwen/Qwen1.5-7B' | 'snorkelai/Snorkel-Mistral-PairRM-DPO' | 'Qwen/Qwen1.5-14B-Chat' | 'Qwen/Qwen1.5-1.8B-Chat' | 'Snowflake/snowflake-arctic-instruct' | 'togethercomputer/llama-2-13b' | 'NousResearch/Nous-Hermes-2-Mixtral-8x7B-SFT' | 'deepseek-ai/deepseek-coder-33b-instruct' | 'togethercomputer/CodeLlama-7b-Instruct' | 'NousResearch/Nous-Hermes-Llama2-13b' | 'lmsys/vicuna-13b-v1.5' | 'togethercomputer/guanaco-13b' | 'togethercomputer/CodeLlama-34b-Instruct' | 'togethercomputer/llama-2-70b' | 'codellama/CodeLlama-13b-Instruct-hf' | 'Qwen/Qwen2-7B' | 'Qwen/Qwen2-1.5B' | 'togethercomputer/CodeLlama-13b-Instruct' | 'meta-llama/Llama-2-13b-hf' | 'togethercomputer/llama-2-13b-chat' | 'huggyllama/llama-30b' | 'NousResearch/Nous-Hermes-2-Mistral-7B-DPO' | 'togethercomputer/alpaca-7b' | 'google/gemma-7b-it' | 'allenai/OLMo-7B' | 'togethercomputer/guanaco-33b' | 'togethercomputer/llama-2-7b-chat' | 'togethercomputer/SOLAR-10.7B-Instruct-v1.0-int4' | 'togethercomputer/guanaco-7b' | 'Open-Orca/Mistral-7B-OpenOrca' | 'Qwen/Qwen1.5-32B' | 'EleutherAI/llemma_7b' | 'NousResearch/Nous-Hermes-llama-2-7b' | 'Qwen/Qwen1.5-32B-Chat' | 'meta-llama/Meta-Llama-3-70B' | 'meta-llama/Llama-3-8b-hf' | 'sentence-transformers/msmarco-bert-base-dot-v5' | 'zero-one-ai/Yi-6B' | 'meta-llama/Meta-Llama-3-8B-Instruct' | 'teknium/OpenHermes-2p5-Mistral-7B' | 'Qwen/Qwen1.5-4B-Chat' | 'wavymulder/Analog-Diffusion' | 'runwayml/stable-diffusion-v1-5' | 'prompthero/openjourney' | 'meta-llama/Llama-2-7b-hf' | 'SG161222/Realistic_Vision_V3.0_VAE' | 'Qwen/Qwen1.5-0.5B-Chat' | 'codellama/CodeLlama-7b-Instruct-hf' | 'google/gemma-2b' | 'mistralai/Mixtral-8x22B' | 'meta-llama/Llama-2-70b-chat-hf' | 'zero-one-ai/Yi-34B-Chat' | 'google/gemma-2-9b' | 'meta-llama/Meta-Llama-3-70B-Instruct' | 'togethercomputer/LLaMA-2-7B-32K' | 'codellama/CodeLlama-70b-Instruct-hf' | 'NousResearch/Hermes-2-Theta-Llama-3-70B' | 'test/test11' | 'stabilityai/stable-diffusion-2-1' | 'microsoft/phi-2' | 'Qwen/Qwen1.5-7B-Chat' | 'cognitivecomputations/dolphin-2.5-mixtral-8x7b' | 'togethercomputer/evo-1-131k-base' | 'togethercomputer/evo-1-8k-base' | 'togethercomputer/llama-2-70b-chat' | 'Qwen/Qwen1.5-14B' | 'carson/ml318br' | 'meta-llama/Meta-Llama-3.1-8B-Instruct-Reference' | 'meta-llama/Meta-Llama-3.1-8B-Reference' | 'gradientai/Llama-3-70B-Instruct-Gradient-1048k' | 'meta-llama/Meta-Llama-3.1-70B-Reference' | 'meta-llama/Meta-Llama-3.1-70B-Instruct-Reference' | 'meta-llama/Llama-3-70b-hf' | 'Qwen/Qwen2-1.5B-Instruct' | 'NousResearch/Nous-Hermes-13b' | 'HuggingFaceH4/zephyr-7b-beta' | 'Austism/chronos-hermes-13b' | 'Qwen/Qwen1.5-1.8B' | 'Qwen/Qwen1.5-72B' | 'lmsys/vicuna-13b-v1.3' | 'huggyllama/llama-13b' | 'garage-bAInd/Platypus2-70B-instruct' | 'allenai/OLMo-7B-Instruct' | 'togethercomputer/Koala-13B' | 'lmsys/vicuna-7b-v1.5';
type TogetherAiConfig = {
    provider: 'TOGETHER_AI';
    name: TogetherAiModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type LMStudioModel = 'qwen2-7b-instruct-4bit' | 'qwen2-math-1.5b' | 'qwen2-0.5b' | 'aya-23-8b' | 'mistral-7b-v0.3' | 'stablecode' | 'cohere-command-r-v01-4bit' | 'command-r' | 'starcoder2-7b' | 'deepseek-math-7b' | 'qwen2.5-coder-14b' | 'qwen2.5-coder-32b' | 'qwen2.5-coder-3b' | 'llama-3.2-3b-instruct-4bit' | 'llama-3.2-1b' | 'llama-3.2-3b' | 'qwen2.5-coder-7b' | 'qwen2.5-14b' | 'yi-coder-9b' | 'hermes-3-llama-3.1-8b' | 'internlm-2.5-20b' | 'llava-v1.5' | 'llama-3.1-8b-instruct-4bit' | 'meta-llama-3.1-8b' | 'mistral-nemo-2407' | 'mistral-nemo-instruct-2407-4bit' | 'gemma-2-2b' | 'mathstral-7b' | 'gemma-2-9b' | 'deepseek-coder-v2-lite-instruct-4bit' | 'smollm-360m-v0.2' | 'phi-3-mini-4k-instruct-4bit' | 'gemma-2-27b' | 'codestral-22b' | 'phi-3.1-mini-128k' | 'deepseek-coder-v2-lite';
type LMStudioConfig = {
    provider: 'LM_STUDIO';
    name: LMStudioModel | (string & {});
    toolChoice?: 'auto' | 'required';
    baseURL: string;
    apiKey?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type BasetenModel = 'llama-3.1-70b-instruct' | 'qwen2.5-7b-math-instruct' | 'qwen2.5-14b-instruct' | 'qwen2.5-32b-coder-instruct' | 'llama-3.1-8b-instruct' | 'llama-3.1-nemetron-70b' | 'llama-3.2-90b-vision-instruct' | 'llama-3.1-405b-instruct' | 'ultravox-v0.4' | 'llama-3.2-1b-vision-instruct' | 'llama-3-70b-instruct' | 'llama-3-8b-instruct' | 'mistral-7b-instruct' | 'qwen2.5-14b-coder-instruct' | 'qwen2.5-7b-coder-instruct' | 'qwen2.5-72b-math-instruct' | 'qwen2.5-72b-instruct' | 'qwen2.5-32b-instruct' | 'qwen2.5-7b-instruct' | 'qwen2.5-3b-instruct' | 'pixtral-12b' | 'phi-3.5-mini-instruct' | 'gemma-2-9b' | 'gemma-2-27b' | 'phi-3-mini-128k-instruct' | 'phi-3-mini-4k-instruct' | 'zephyr-7b-alpha' | 'mixtral-8x7b-instruct' | 'mixtral-8x22b';
type BaseTenConfig = {
    provider: 'BASETEN';
    name: BasetenModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type FireworksModel = 'llama-3.1-405b-instruct' | 'llama-3.1-70b-instruct' | 'llama-3.1-8b-instruct' | 'llama-3.2-3b-instruct';
type FireworksConfig = {
    provider: 'FIREWORKS';
    name: FireworksModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type MistralModel = 'pixtral-large-latest' | 'mistral-large-latest' | 'mistral-small-latest' | 'ministral-3b-latest' | 'ministral-8b-latest' | 'pixtral-12b-2409';
type MistralConfig = {
    provider: 'MISTRAL';
    name: MistralModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type XGrokModel = 'grok-beta' | 'grok-vision-beta';
type XGrokConfig = {
    provider: 'X_GROK';
    name: XGrokModel | (string & {});
    toolChoice?: 'auto' | 'required';
    apiKey?: string;
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type CustomModelConfig = {
    model: LanguageModelV1 | (() => Promise<LanguageModelV1>);
    provider: string;
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type CohereModel = 'command-r-plus';
type CohereConfig = {
    provider: 'COHERE';
    name: CohereModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type AzureModel = 'gpt-35-turbo-instruct';
type AzureConfig = {
    provider: 'AZURE';
    name: AzureModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    headers?: Record<string, string>;
    apiVersion?: string;
    baseURL?: string;
    fetch?: typeof globalThis.fetch;
};
type DeepseekModel = 'deepseek-chat' | 'deepseek-reasoner';
type DeepseekConfig = {
    provider: 'DEEPSEEK';
    name: DeepseekModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type AmazonModel = 'amazon-titan-tg1-large' | 'amazon-titan-text-express-v1' | 'anthropic-claude-3-5-sonnet-20241022-v2:0' | 'anthropic-claude-3-5-sonnet-20240620-v1:0' | 'anthropic-claude-3-5-haiku-20241022-v1:0' | 'anthropic-claude-3-opus-20240229-v1:0' | 'anthropic-claude-3-sonnet-20240229-v1:0' | 'anthropic-claude-3-haiku-20240307-v1:0' | 'anthropic-claude-v2:1' | 'cohere-command-r-v1:0' | 'cohere-command-r-plus-v1:0' | 'meta-llama2-13b-chat-v1' | 'meta-llama2-70b-chat-v1' | 'meta-llama3-8b-instruct-v1:0' | 'meta-llama3-70b-instruct-v1:0' | 'meta-llama3-1-8b-instruct-v1:0' | 'meta-llama3-1-70b-instruct-v1:0' | 'meta-llama3-1-405b-instruct-v1:0' | 'meta-llama3-2-1b-instruct-v1:0' | 'meta-llama3-2-3b-instruct-v1:0' | 'meta-llama3-2-11b-instruct-v1:0' | 'meta-llama3-2-90b-instruct-v1:0' | 'mistral-mistral-7b-instruct-v0:2' | 'mistral-mixtral-8x7b-instruct-v0:1' | 'mistral-mistral-large-2402-v1:0' | 'mistral-mistral-small-2402-v1:0';
type AmazonConfig = {
    provider: 'AMAZON';
    name: AmazonModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type AnthropicVertexModel = 'claude-3-5-sonnet@20240620' | 'claude-3-opus@20240229' | 'claude-3-sonnet@20240229' | 'claude-3-haiku@20240307';
type AnthropicVertexConfig = {
    provider: 'ANTHROPIC_VERTEX';
    name: AnthropicVertexModel | (string & {});
    apiKey?: string;
    toolChoice?: 'auto' | 'required';
    baseURL?: string;
    headers?: Record<string, string>;
    fetch?: typeof globalThis.fetch;
};
type BuiltInModelConfig = OpenAIConfig | AnthropicConfig | GroqConfig | GoogleConfig | PerplexityConfig | TogetherAiConfig | LMStudioConfig | BaseTenConfig | FireworksConfig | MistralConfig | XGrokConfig | CohereConfig | AzureConfig | AmazonConfig | AnthropicVertexConfig | DeepseekConfig;
type ModelConfig = BuiltInModelConfig | CustomModelConfig;
type LLMProvider = BuiltInModelConfig['provider'];
type BaseStructuredOutputType = 'string' | 'number' | 'boolean' | 'date';
type StructuredOutputType = 'array' | 'string' | 'number' | 'object' | 'boolean' | 'date';
type StructuredOutputArrayItem = {
    type: BaseStructuredOutputType;
} | {
    type: 'object';
    items: StructuredOutput;
};
type StructuredOutput = {
    [key: string]: {
        type: BaseStructuredOutputType;
    } | {
        type: 'object';
        items: StructuredOutput;
    } | {
        type: 'array';
        items: StructuredOutputArrayItem;
    };
};
type GenerateReturn<Z extends ZodSchema | JSONSchema7 | undefined = undefined> = Z extends undefined ? GenerateTextResult<any, any> : GenerateObjectResult<any>;
type StreamReturn<Z extends ZodSchema | JSONSchema7 | undefined = undefined> = Z extends undefined ? StreamTextResult<any, any> : StreamObjectResult<any, any, any>;
type OutputType = 'text' | StructuredOutput;
type LLMStreamOptions<Z extends ZodSchema | JSONSchema7 | undefined = undefined> = {
    runId?: string;
    onFinish?: (result: string) => Promise<void> | void;
    onStepFinish?: (step: string) => void;
    maxSteps?: number;
    tools?: ToolsInput;
    convertedTools?: Record<string, CoreTool>;
    output?: OutputType | Z;
    temperature?: number;
};
type LLMTextOptions = {
    tools?: ToolsInput;
    convertedTools?: Record<string, CoreTool>;
    messages: CoreMessage[];
    onStepFinish?: (step: string) => void;
    toolChoice?: 'auto' | 'required';
    maxSteps?: number;
    temperature?: number;
} & Run;
type LLMTextObjectOptions<T> = LLMTextOptions & {
    structuredOutput: JSONSchema7 | z.ZodType<T> | StructuredOutput;
};
type LLMInnerStreamOptions = {
    tools?: ToolsInput;
    convertedTools?: Record<string, CoreTool>;
    messages: CoreMessage[];
    onStepFinish?: (step: string) => void;
    onFinish?: (result: string) => Promise<void> | void;
    maxSteps?: number;
    temperature?: number;
} & Run;
type LLMStreamObjectOptions<T> = LLMInnerStreamOptions & {
    structuredOutput: JSONSchema7 | z.ZodType<T> | StructuredOutput;
};

declare class MastraLLMBase extends MastraBase {
    #private;
    constructor({ name }: {
        name: string;
    });
    convertToMessages(messages: string | string[] | CoreMessage$1[]): CoreMessage$1[];
    __registerPrimitives(p: MastraPrimitives): void;
    __text(input: LLMTextOptions): Promise<GenerateTextResult<any, any>>;
    __textObject<T>(input: LLMTextObjectOptions<T>): Promise<GenerateObjectResult<T>>;
    generate<Z extends ZodSchema | JSONSchema7 | undefined = undefined>(messages: string | string[] | CoreMessage$1[], options?: LLMStreamOptions<Z>): Promise<GenerateReturn<Z>>;
    __stream(input: LLMInnerStreamOptions): Promise<StreamTextResult<any, any>>;
    __streamObject<T>(input: LLMStreamObjectOptions<T>): Promise<StreamObjectResult<DeepPartial<T>, T, never>>;
    stream<Z extends ZodSchema | JSONSchema7 | undefined = undefined>(messages: string | string[] | CoreMessage$1[], options?: LLMStreamOptions<Z>): Promise<StreamReturn<Z>>;
}

interface StorageColumn {
    type: 'text' | 'timestamp' | 'uuid' | 'jsonb' | 'integer';
    primaryKey?: boolean;
    nullable?: boolean;
    references?: {
        table: string;
        column: string;
    };
}
interface WorkflowRow {
    workflow_name: string;
    run_id: string;
    snapshot: WorkflowRunState;
    created_at: Date;
    updated_at: Date;
}
type StorageGetMessagesArg = {
    threadId: string;
    selectBy?: {
        vectorSearchString?: string;
        last?: number | false;
        include?: {
            id: string;
            withPreviousMessages?: number;
            withNextMessages?: number;
        }[];
    };
    threadConfig?: MemoryConfig;
};

type TABLE_NAMES = typeof MastraStorage.TABLE_WORKFLOW_SNAPSHOT | typeof MastraStorage.TABLE_EVALS | typeof MastraStorage.TABLE_MESSAGES | typeof MastraStorage.TABLE_THREADS;
declare abstract class MastraStorage extends MastraBase {
    static readonly TABLE_WORKFLOW_SNAPSHOT = "mastra_workflow_snapshot";
    static readonly TABLE_EVALS = "mastra_evals";
    static readonly TABLE_MESSAGES = "mastra_messages";
    static readonly TABLE_THREADS = "mastra_threads";
    hasInit: boolean;
    constructor({ name }: {
        name: string;
    });
    abstract createTable({ tableName }: {
        tableName: TABLE_NAMES;
        schema: Record<string, StorageColumn>;
    }): Promise<void>;
    abstract clearTable({ tableName }: {
        tableName: TABLE_NAMES;
    }): Promise<void>;
    abstract insert({ tableName, record }: {
        tableName: TABLE_NAMES;
        record: Record<string, any>;
    }): Promise<void>;
    abstract load<R>({ tableName, keys }: {
        tableName: TABLE_NAMES;
        keys: Record<string, string>;
    }): Promise<R | null>;
    abstract getThreadById({ threadId }: {
        threadId: string;
    }): Promise<StorageThreadType | null>;
    __getThreadById({ threadId }: {
        threadId: string;
    }): Promise<StorageThreadType | null>;
    abstract getThreadsByResourceId({ resourceId }: {
        resourceId: string;
    }): Promise<StorageThreadType[]>;
    __getThreadsByResourceId({ resourceId }: {
        resourceId: string;
    }): Promise<StorageThreadType[]>;
    abstract saveThread({ thread }: {
        thread: StorageThreadType;
    }): Promise<StorageThreadType>;
    __saveThread({ thread }: {
        thread: StorageThreadType;
    }): Promise<StorageThreadType>;
    abstract updateThread({ id, title, metadata, }: {
        id: string;
        title: string;
        metadata: Record<string, unknown>;
    }): Promise<StorageThreadType>;
    __updateThread({ id, title, metadata, }: {
        id: string;
        title: string;
        metadata: Record<string, unknown>;
    }): Promise<StorageThreadType>;
    abstract deleteThread({ threadId }: {
        threadId: string;
    }): Promise<void>;
    __deleteThread({ threadId }: {
        threadId: string;
    }): Promise<void>;
    abstract getMessages({ threadId, selectBy, threadConfig }: StorageGetMessagesArg): Promise<MessageType[]>;
    __getMessages({ threadId, selectBy, threadConfig }: StorageGetMessagesArg): Promise<MessageType[]>;
    abstract saveMessages({ messages }: {
        messages: MessageType[];
    }): Promise<MessageType[]>;
    __saveMessages({ messages }: {
        messages: MessageType[];
    }): Promise<MessageType[]>;
    init(): Promise<void>;
    persistWorkflowSnapshot({ workflowName, runId, snapshot, }: {
        workflowName: string;
        runId: string;
        snapshot: WorkflowRunState;
    }): Promise<void>;
    loadWorkflowSnapshot({ workflowName, runId, }: {
        workflowName: string;
        runId: string;
    }): Promise<WorkflowRunState | null>;
}

type AiMessageType = Message;
type MessageType = {
    id: string;
    content: UserContent | AssistantContent | ToolContent;
    role: 'system' | 'user' | 'assistant' | 'tool';
    createdAt: Date;
    threadId: string;
    toolCallIds?: string[];
    toolCallArgs?: Record<string, unknown>[];
    toolNames?: string[];
    type: 'text' | 'tool-call' | 'tool-result';
};
type StorageThreadType = {
    id: string;
    title?: string;
    resourceId: string;
    createdAt: Date;
    updatedAt: Date;
    metadata?: Record<string, unknown>;
};
type MessageResponse<T extends 'raw' | 'core_message'> = {
    raw: MessageType[];
    core_message: CoreMessage$1[];
}[T];
type MemoryConfig = {
    lastMessages?: number | false;
    semanticRecall?: boolean | {
        topK: number;
        messageRange: number | {
            before: number;
            after: number;
        };
    };
    workingMemory?: {
        enabled: boolean;
        template?: string;
    };
};
type SharedMemoryConfig = {
    storage: MastraStorage;
    options?: MemoryConfig;
    vector?: MastraVector;
    embedding?: EmbeddingOptions;
} | {
    storage: MastraStorage;
    options?: MemoryConfig;
    vector: MastraVector;
    embedding: EmbeddingOptions;
};
/**
 * Abstract Memory class that defines the interface for storing and retrieving
 * conversation threads and messages.
 */
declare abstract class MastraMemory extends MastraBase {
    MAX_CONTEXT_TOKENS?: number;
    storage: MastraStorage;
    vector?: MastraVector;
    embedding?: EmbeddingOptions;
    protected threadConfig: MemoryConfig;
    constructor(config: {
        name: string;
    } & SharedMemoryConfig);
    /**
     * Get a system message to inject into the conversation.
     * This will be called before each conversation turn.
     * Implementations can override this to inject custom system messages.
     */
    getSystemMessage(_input: {
        threadId: string;
        memoryConfig?: MemoryConfig;
    }): Promise<string | null>;
    protected parseEmbeddingOptions(): EmbeddingOptions;
    protected getMergedThreadConfig(config?: MemoryConfig): MemoryConfig;
    abstract rememberMessages({ threadId, vectorMessageSearch, config, }: {
        threadId: string;
        vectorMessageSearch?: string;
        config?: MemoryConfig;
    }): Promise<{
        messages: CoreMessage$1[];
        uiMessages: AiMessageType[];
    }>;
    estimateTokens(text: string): number;
    protected parseMessages(messages: MessageType[]): CoreMessage$1[];
    protected convertToUIMessages(messages: MessageType[]): AiMessageType[];
    /**
     * Retrieves a specific thread by its ID
     * @param threadId - The unique identifier of the thread
     * @returns Promise resolving to the thread or null if not found
     */
    abstract getThreadById({ threadId }: {
        threadId: string;
    }): Promise<StorageThreadType | null>;
    abstract getThreadsByResourceId({ resourceId }: {
        resourceId: string;
    }): Promise<StorageThreadType[]>;
    /**
     * Saves or updates a thread
     * @param thread - The thread data to save
     * @returns Promise resolving to the saved thread
     */
    abstract saveThread({ thread, memoryConfig, }: {
        thread: StorageThreadType;
        memoryConfig?: MemoryConfig;
    }): Promise<StorageThreadType>;
    /**
     * Saves messages to a thread
     * @param messages - Array of messages to save
     * @returns Promise resolving to the saved messages
     */
    abstract saveMessages({ messages, memoryConfig, }: {
        messages: MessageType[];
        memoryConfig: MemoryConfig | undefined;
    }): Promise<MessageType[]>;
    /**
     * Retrieves all messages for a specific thread
     * @param threadId - The unique identifier of the thread
     * @returns Promise resolving to array of messages and uiMessages
     */
    abstract query({ threadId, selectBy, }: StorageGetMessagesArg): Promise<{
        messages: CoreMessage$1[];
        uiMessages: AiMessageType[];
    }>;
    /**
     * Helper method to create a new thread
     * @param title - Optional title for the thread
     * @param metadata - Optional metadata for the thread
     * @returns Promise resolving to the created thread
     */
    createThread({ threadId, resourceId, title, metadata, memoryConfig, }: {
        resourceId: string;
        threadId?: string;
        title?: string;
        metadata?: Record<string, unknown>;
        memoryConfig?: MemoryConfig;
    }): Promise<StorageThreadType>;
    /**
     * Helper method to delete a thread
     * @param threadId - the id of the thread to delete
     */
    abstract deleteThread(threadId: string): Promise<void>;
    /**
     * Helper method to add a single message to a thread
     * @param threadId - The thread to add the message to
     * @param content - The message content
     * @param role - The role of the message sender
     * @param type - The type of the message
     * @param toolNames - Optional array of tool names that were called
     * @param toolCallArgs - Optional array of tool call arguments
     * @param toolCallIds - Optional array of tool call ids
     * @returns Promise resolving to the saved message
     */
    addMessage({ threadId, config, content, role, type, toolNames, toolCallArgs, toolCallIds, }: {
        threadId: string;
        config?: MemoryConfig;
        content: UserContent | AssistantContent;
        role: 'user' | 'assistant';
        type: 'text' | 'tool-call' | 'tool-result';
        toolNames?: string[];
        toolCallArgs?: Record<string, unknown>[];
        toolCallIds?: string[];
    }): Promise<MessageType>;
    /**
     * Generates a unique identifier
     * @returns A unique string ID
     */
    generateId(): string;
}

type ToolsetsInput = Record<string, Record<string, ToolAction<any, any, any, any>>>;
type ToolsInput = Record<string, ToolAction<any, any, any, any>>;
interface AgentConfig<TTools extends Record<string, ToolAction<any, any, any, any>> = Record<string, ToolAction<any, any, any, any>>, TMetrics extends Record<string, Metric> = Record<string, Metric>> {
    name: string;
    instructions: string;
    model?: ModelConfig;
    llm?: MastraLLMBase;
    tools?: TTools;
    mastra?: MastraPrimitives;
    metrics?: TMetrics;
    memory?: MastraMemory;
}
interface AgentGenerateOptions<Z extends ZodSchema | JSONSchema7 | undefined = undefined> {
    toolsets?: ToolsetsInput;
    resourceId?: string;
    context?: CoreMessage[];
    threadId?: string;
    memoryOptions?: MemoryConfig;
    runId?: string;
    onStepFinish?: (step: string) => void;
    maxSteps?: number;
    output?: OutputType | Z;
    temperature?: number;
}
interface AgentStreamOptions<Z extends ZodSchema | JSONSchema7 | undefined = undefined> {
    toolsets?: ToolsetsInput;
    resourceId?: string;
    context?: CoreMessage[];
    threadId?: string;
    memoryOptions?: MemoryConfig;
    runId?: string;
    onFinish?: (result: string) => Promise<void> | void;
    onStepFinish?: (step: string) => void;
    maxSteps?: number;
    output?: OutputType | Z;
    temperature?: number;
}

declare class LLM extends MastraBase {
    #private;
    constructor({ model }: {
        model: ModelConfig;
    });
    __registerPrimitives(p: MastraPrimitives): void;
    getModelType(): string;
    createOpenAICompatibleModel({ baseURL, apiKey, defaultModelName, modelName, fetch, }: {
        baseURL: string;
        apiKey: string;
        defaultModelName: string;
        modelName?: string;
        fetch?: typeof globalThis.fetch;
    }): LanguageModelV1;
    createModelDef({ model, }: {
        model: {
            type: string;
            name?: string;
            toolChoice?: 'auto' | 'required';
            baseURL?: string;
            fetch?: typeof globalThis.fetch;
            apiVersion?: string;
            headers?: Record<string, string>;
            apiKey?: string;
        };
    }): LanguageModelV1;
    getParams({ tools, resultTool, model, }: {
        tools: Record<string, CoreTool>;
        resultTool?: {
            description: string;
            parameters: ZodSchema;
        };
        model: ({
            type: string;
            name?: string;
            toolChoice?: 'auto' | 'required';
            baseURL?: string;
            apiKey?: string;
            apiVersion?: string;
            headers?: Record<string, string>;
            fetch?: typeof globalThis.fetch;
        } & GoogleGenerativeAISettings) | CustomModelConfig;
    }): Promise<{
        toolsConverted: Record<string, CoreTool$1>;
        modelDef: LanguageModelV1;
        answerTool: {};
        toolChoice: "auto" | "required";
    }>;
    convertTools(tools?: ToolsInput): Record<string, CoreTool>;
    private convertToMessages;
    generate<Z extends ZodSchema | JSONSchema7 | undefined = undefined>(messages: string | string[] | CoreMessage$1[], { maxSteps, onStepFinish, tools, convertedTools, runId, output, temperature, }?: LLMStreamOptions<Z>): Promise<GenerateReturn<Z>>;
    stream<Z extends ZodSchema | JSONSchema7 | undefined = undefined>(messages: string | string[] | CoreMessage$1[], { maxSteps, onFinish, onStepFinish, tools, convertedTools, runId, output, temperature, }?: LLMStreamOptions<Z>): Promise<StreamReturn<Z>>;
    __text({ messages, onStepFinish, maxSteps, tools, runId, convertedTools, temperature }: LLMTextOptions): Promise<ai.GenerateTextResult<{}, never>>;
    __getNamedModel(): {
        type: string;
        name: (string & {}) | "gpt-4" | "gpt-4-turbo" | "gpt-3.5-turbo" | "gpt-4o" | "gpt-4o-mini" | "o1" | "o1-mini" | "o1-preview" | "gemini-1.5-pro-latest" | "gemini-1.5-pro" | "gemini-1.5-flash-latest" | "gemini-1.5-flash" | "gemini-2.0-flash-exp-latest" | "gemini-2.0-flash-thinking-exp-1219" | "gemini-exp-1206" | "claude-3-5-sonnet-20241022" | "claude-3-5-sonnet-20240620" | "claude-3-5-haiku-20241022" | "claude-3-opus-20240229" | "claude-3-sonnet-20240229" | "claude-3-haiku-20240307" | "llama3-groq-70b-8192-tool-use-preview" | "llama3-groq-8b-8192-tool-use-preview" | "gemma2-9b-it" | "gemma-7b-it" | "llama-3.1-sonar-small-128k-online" | "llama-3.1-sonar-large-128k-online" | "llama-3.1-sonar-huge-128k-online" | "llama-3.1-sonar-small-128k-chat" | "llama-3.1-sonar-large-128k-chat" | "llama-3.1-8b-instruct" | "llama-3.1-70b-instruct" | "sonar" | "sonar-pro" | "codellama/CodeLlama-34b-Instruct-hf" | "upstage/SOLAR-10.7B-Instruct-v1.0" | "mistralai/Mixtral-8x7B-v0.1" | "WhereIsAI/UAE-Large-V1" | "black-forest-labs/FLUX.1-depth" | "togethercomputer/m2-bert-80M-32k-retrieval" | "black-forest-labs/FLUX.1-canny" | "black-forest-labs/FLUX.1-dev" | "black-forest-labs/FLUX.1-redux" | "BAAI/bge-large-en-v1.5" | "meta-llama/Llama-3.2-90B-Vision-Instruct-Turbo" | "togethercomputer/Llama-3-8b-chat-hf-int4" | "stabilityai/stable-diffusion-xl-base-1.0" | "Gryphe/MythoMax-L2-13b" | "meta-llama/Meta-Llama-3-8B" | "mistralai/Mixtral-8x7B-Instruct-v0.1" | "deepseek-ai/deepseek-llm-67b-chat" | "togethercomputer/m2-bert-80M-8k-retrieval" | "llava-hf/llava-v1.6-mistral-7b-hf" | "Qwen/Qwen2.5-Coder-32B-Instruct" | "databricks/dbrx-instruct" | "meta-llama/Meta-Llama-3.1-405B-Instruct-Lite-Pro" | "meta-llama/Meta-Llama-3-8B-Instruct-Lite" | "scb10x/scb10x-llama3-typhoon-v1-5-8b-instruct" | "microsoft/WizardLM-2-8x22B" | "BAAI/bge-base-en-v1.5" | "togethercomputer/m2-bert-80M-2k-retrieval" | "google/gemma-2b-it" | "meta-llama/Llama-2-70b-hf" | "mistralai/Mistral-7B-Instruct-v0.2" | "meta-llama/LlamaGuard-2-8b" | "mistralai/Mistral-7B-Instruct-v0.1" | "mistralai/Mistral-7B-v0.1" | "black-forest-labs/FLUX.1-pro" | "black-forest-labs/FLUX.1-schnell" | "meta-llama/Meta-Llama-3.1-405B-Instruct-Turbo" | "nvidia/Llama-3.1-Nemotron-70B-Instruct-HF" | "mistralai/Mistral-7B-Instruct-v0.3" | "Meta-Llama/Llama-Guard-7b" | "meta-llama/Meta-Llama-3-70B-Instruct-Turbo" | "meta-llama/Meta-Llama-3-70B-Instruct-Lite" | "google/gemma-2-27b-it" | "meta-llama/Llama-3-8b-chat-hf" | "Qwen/Qwen2.5-72B-Instruct-Turbo" | "Salesforce/Llama-Rank-V1" | "meta-llama/Llama-Guard-3-11B-Vision-Turbo" | "google/gemma-2-9b-it" | "meta-llama/Llama-3.2-11B-Vision-Instruct-Turbo" | "NousResearch/Nous-Hermes-2-Mixtral-8x7B-DPO" | "meta-llama/Llama-3-70b-chat-hf" | "Qwen/Qwen2.5-7B-Instruct-Turbo" | "scb10x/scb10x-llama3-typhoon-v1-5x-4f316" | "meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo" | "Gryphe/MythoMax-L2-13b-Lite" | "black-forest-labs/FLUX.1-schnell-Free" | "meta-llama/Llama-2-7b-chat-hf" | "meta-llama/Meta-Llama-Guard-3-8B" | "togethercomputer/Llama-3-8b-chat-hf-int8" | "meta-llama/Llama-Vision-Free" | "Qwen/Qwen2-72B-Instruct" | "mistralai/Mixtral-8x22B-Instruct-v0.1" | "black-forest-labs/FLUX.1.1-pro" | "meta-llama/Llama-3.2-3B-Instruct-Turbo" | "meta-llama/Llama-2-13b-chat-hf" | "meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo" | "Nexusflow/NexusRaven-V2-13B" | "bert-base-uncased" | "WizardLM/WizardLM-13B-V1.2" | "google/gemma-7b" | "togethercomputer/Koala-7B" | "zero-one-ai/Yi-34B" | "togethercomputer/StripedHyena-Hessian-7B" | "teknium/OpenHermes-2-Mistral-7B" | "Qwen/Qwen2-7B-Instruct" | "togethercomputer/guanaco-65b" | "togethercomputer/llama-2-7b" | "hazyresearch/M2-BERT-2k-Retrieval-Encoder-V1" | "huggyllama/llama-7b" | "Undi95/ReMM-SLERP-L2-13B" | "NousResearch/Nous-Capybara-7B-V1p9" | "lmsys/vicuna-7b-v1.3" | "Undi95/Toppy-M-7B" | "Qwen/Qwen2-72B" | "NousResearch/Nous-Hermes-Llama2-70b" | "WizardLM/WizardLM-70B-V1.0" | "huggyllama/llama-65b" | "lmsys/vicuna-13b-v1.5-16k" | "openchat/openchat-3.5-1210" | "Qwen/Qwen1.5-0.5B" | "Qwen/Qwen1.5-4B" | "Qwen/Qwen1.5-7B" | "snorkelai/Snorkel-Mistral-PairRM-DPO" | "Qwen/Qwen1.5-14B-Chat" | "Qwen/Qwen1.5-1.8B-Chat" | "Snowflake/snowflake-arctic-instruct" | "togethercomputer/llama-2-13b" | "NousResearch/Nous-Hermes-2-Mixtral-8x7B-SFT" | "deepseek-ai/deepseek-coder-33b-instruct" | "togethercomputer/CodeLlama-7b-Instruct" | "NousResearch/Nous-Hermes-Llama2-13b" | "lmsys/vicuna-13b-v1.5" | "togethercomputer/guanaco-13b" | "togethercomputer/CodeLlama-34b-Instruct" | "togethercomputer/llama-2-70b" | "codellama/CodeLlama-13b-Instruct-hf" | "Qwen/Qwen2-7B" | "Qwen/Qwen2-1.5B" | "togethercomputer/CodeLlama-13b-Instruct" | "meta-llama/Llama-2-13b-hf" | "togethercomputer/llama-2-13b-chat" | "huggyllama/llama-30b" | "NousResearch/Nous-Hermes-2-Mistral-7B-DPO" | "togethercomputer/alpaca-7b" | "google/gemma-7b-it" | "allenai/OLMo-7B" | "togethercomputer/guanaco-33b" | "togethercomputer/llama-2-7b-chat" | "togethercomputer/SOLAR-10.7B-Instruct-v1.0-int4" | "togethercomputer/guanaco-7b" | "Open-Orca/Mistral-7B-OpenOrca" | "Qwen/Qwen1.5-32B" | "EleutherAI/llemma_7b" | "NousResearch/Nous-Hermes-llama-2-7b" | "Qwen/Qwen1.5-32B-Chat" | "meta-llama/Meta-Llama-3-70B" | "meta-llama/Llama-3-8b-hf" | "sentence-transformers/msmarco-bert-base-dot-v5" | "zero-one-ai/Yi-6B" | "meta-llama/Meta-Llama-3-8B-Instruct" | "teknium/OpenHermes-2p5-Mistral-7B" | "Qwen/Qwen1.5-4B-Chat" | "wavymulder/Analog-Diffusion" | "runwayml/stable-diffusion-v1-5" | "prompthero/openjourney" | "meta-llama/Llama-2-7b-hf" | "SG161222/Realistic_Vision_V3.0_VAE" | "Qwen/Qwen1.5-0.5B-Chat" | "codellama/CodeLlama-7b-Instruct-hf" | "google/gemma-2b" | "mistralai/Mixtral-8x22B" | "meta-llama/Llama-2-70b-chat-hf" | "zero-one-ai/Yi-34B-Chat" | "google/gemma-2-9b" | "meta-llama/Meta-Llama-3-70B-Instruct" | "togethercomputer/LLaMA-2-7B-32K" | "codellama/CodeLlama-70b-Instruct-hf" | "NousResearch/Hermes-2-Theta-Llama-3-70B" | "test/test11" | "stabilityai/stable-diffusion-2-1" | "microsoft/phi-2" | "Qwen/Qwen1.5-7B-Chat" | "cognitivecomputations/dolphin-2.5-mixtral-8x7b" | "togethercomputer/evo-1-131k-base" | "togethercomputer/evo-1-8k-base" | "togethercomputer/llama-2-70b-chat" | "Qwen/Qwen1.5-14B" | "carson/ml318br" | "meta-llama/Meta-Llama-3.1-8B-Instruct-Reference" | "meta-llama/Meta-Llama-3.1-8B-Reference" | "gradientai/Llama-3-70B-Instruct-Gradient-1048k" | "meta-llama/Meta-Llama-3.1-70B-Reference" | "meta-llama/Meta-Llama-3.1-70B-Instruct-Reference" | "meta-llama/Llama-3-70b-hf" | "Qwen/Qwen2-1.5B-Instruct" | "NousResearch/Nous-Hermes-13b" | "HuggingFaceH4/zephyr-7b-beta" | "Austism/chronos-hermes-13b" | "Qwen/Qwen1.5-1.8B" | "Qwen/Qwen1.5-72B" | "lmsys/vicuna-13b-v1.3" | "huggyllama/llama-13b" | "garage-bAInd/Platypus2-70B-instruct" | "allenai/OLMo-7B-Instruct" | "togethercomputer/Koala-13B" | "lmsys/vicuna-7b-v1.5" | "qwen2-7b-instruct-4bit" | "qwen2-math-1.5b" | "qwen2-0.5b" | "aya-23-8b" | "mistral-7b-v0.3" | "stablecode" | "cohere-command-r-v01-4bit" | "command-r" | "starcoder2-7b" | "deepseek-math-7b" | "qwen2.5-coder-14b" | "qwen2.5-coder-32b" | "qwen2.5-coder-3b" | "llama-3.2-3b-instruct-4bit" | "llama-3.2-1b" | "llama-3.2-3b" | "qwen2.5-coder-7b" | "qwen2.5-14b" | "yi-coder-9b" | "hermes-3-llama-3.1-8b" | "internlm-2.5-20b" | "llava-v1.5" | "llama-3.1-8b-instruct-4bit" | "meta-llama-3.1-8b" | "mistral-nemo-2407" | "mistral-nemo-instruct-2407-4bit" | "gemma-2-2b" | "mathstral-7b" | "gemma-2-9b" | "deepseek-coder-v2-lite-instruct-4bit" | "smollm-360m-v0.2" | "phi-3-mini-4k-instruct-4bit" | "gemma-2-27b" | "codestral-22b" | "phi-3.1-mini-128k" | "deepseek-coder-v2-lite" | "qwen2.5-7b-math-instruct" | "qwen2.5-14b-instruct" | "qwen2.5-32b-coder-instruct" | "llama-3.1-nemetron-70b" | "llama-3.2-90b-vision-instruct" | "llama-3.1-405b-instruct" | "ultravox-v0.4" | "llama-3.2-1b-vision-instruct" | "llama-3-70b-instruct" | "llama-3-8b-instruct" | "mistral-7b-instruct" | "qwen2.5-14b-coder-instruct" | "qwen2.5-7b-coder-instruct" | "qwen2.5-72b-math-instruct" | "qwen2.5-72b-instruct" | "qwen2.5-32b-instruct" | "qwen2.5-7b-instruct" | "qwen2.5-3b-instruct" | "pixtral-12b" | "phi-3.5-mini-instruct" | "phi-3-mini-128k-instruct" | "phi-3-mini-4k-instruct" | "zephyr-7b-alpha" | "mixtral-8x7b-instruct" | "mixtral-8x22b" | "llama-3.2-3b-instruct" | "pixtral-large-latest" | "mistral-large-latest" | "mistral-small-latest" | "ministral-3b-latest" | "ministral-8b-latest" | "pixtral-12b-2409" | "grok-beta" | "grok-vision-beta" | "command-r-plus" | "gpt-35-turbo-instruct" | "deepseek-chat" | "deepseek-reasoner" | "amazon-titan-tg1-large" | "amazon-titan-text-express-v1" | "anthropic-claude-3-5-sonnet-20241022-v2:0" | "anthropic-claude-3-5-sonnet-20240620-v1:0" | "anthropic-claude-3-5-haiku-20241022-v1:0" | "anthropic-claude-3-opus-20240229-v1:0" | "anthropic-claude-3-sonnet-20240229-v1:0" | "anthropic-claude-3-haiku-20240307-v1:0" | "anthropic-claude-v2:1" | "cohere-command-r-v1:0" | "cohere-command-r-plus-v1:0" | "meta-llama2-13b-chat-v1" | "meta-llama2-70b-chat-v1" | "meta-llama3-8b-instruct-v1:0" | "meta-llama3-70b-instruct-v1:0" | "meta-llama3-1-8b-instruct-v1:0" | "meta-llama3-1-70b-instruct-v1:0" | "meta-llama3-1-405b-instruct-v1:0" | "meta-llama3-2-1b-instruct-v1:0" | "meta-llama3-2-3b-instruct-v1:0" | "meta-llama3-2-11b-instruct-v1:0" | "meta-llama3-2-90b-instruct-v1:0" | "mistral-mistral-7b-instruct-v0:2" | "mistral-mixtral-8x7b-instruct-v0:1" | "mistral-mistral-large-2402-v1:0" | "mistral-mistral-small-2402-v1:0" | "claude-3-5-sonnet@20240620" | "claude-3-opus@20240229" | "claude-3-sonnet@20240229" | "claude-3-haiku@20240307";
        toolChoice: "auto" | "required" | undefined;
        apiKey: string | undefined;
        baseURL: string | undefined;
        headers: Record<string, string> | undefined;
        fetch: typeof fetch | undefined;
    };
    __textObject<T>({ messages, onStepFinish, maxSteps, tools, convertedTools, structuredOutput, runId, temperature, }: LLMTextObjectOptions<T>): Promise<ai.GenerateObjectResult<T>>;
    __stream({ messages, onStepFinish, onFinish, maxSteps, tools, runId, convertedTools, temperature, }: LLMInnerStreamOptions): Promise<ai.StreamTextResult<{}, never>>;
    __streamObject<T>({ messages, onStepFinish, onFinish, maxSteps, tools, convertedTools, structuredOutput, runId, temperature, }: LLMStreamObjectOptions<T>): Promise<ai.StreamObjectResult<ai.DeepPartial<T>, T, never>>;
}

type MastraPrimitives = {
    logger?: Logger;
    telemetry?: Telemetry;
    engine?: MastraEngine;
    storage?: MastraStorage;
    agents?: Record<string, Agent>;
    tts?: Record<string, MastraTTS>;
    vectors?: Record<string, MastraVector>;
    memory?: MastraMemory;
    llm?: (model: ModelConfig) => LLM;
};
interface IExecutionContext<TSchemaIn extends z.ZodSchema | undefined = undefined, TContext extends WorkflowContext = WorkflowContext> {
    context: TSchemaIn extends z.ZodSchema ? z.infer<TSchemaIn> & {
        machineContext?: TContext;
    } : {
        machineContext?: TContext;
    };
    runId?: string;
    mastra?: MastraPrimitives;
    suspend: () => Promise<void>;
}
interface IAction<TId extends string, TSchemaIn extends z.ZodSchema | undefined, TSchemaOut extends z.ZodSchema | undefined, TContext extends IExecutionContext<TSchemaIn>> {
    id: TId;
    description?: string;
    inputSchema?: TSchemaIn;
    outputSchema?: TSchemaOut;
    mastra?: MastraPrimitives;
    payload?: TSchemaIn extends z.ZodSchema ? Partial<z.infer<TSchemaIn>> : unknown;
    execute: (context: TContext) => Promise<TSchemaOut extends z.ZodSchema ? z.infer<TSchemaOut> : unknown>;
    [key: string]: any;
}

declare class Agent<TTools extends Record<string, ToolAction<any, any, any, any>> = Record<string, ToolAction<any, any, any, any>>, TMetrics extends Record<string, Metric> = Record<string, Metric>> extends MastraBase {
    #private;
    name: string;
    readonly llm: LLM | MastraLLMBase;
    readonly instructions: string;
    readonly model?: ModelConfig;
    tools: TTools;
    metrics: TMetrics;
    constructor(config: AgentConfig<TTools, TMetrics>);
    hasOwnMemory(): boolean;
    getMemory(): MastraMemory | undefined;
    __registerPrimitives(p: MastraPrimitives): void;
    /**
     * Set the concrete tools for the agent
     * @param tools
     */
    __setTools(tools: TTools): void;
    generateTitleFromUserMessage({ message }: {
        message: CoreUserMessage$1;
    }): Promise<string>;
    getMostRecentUserMessage(messages: Array<CoreMessage$1>): CoreUserMessage$1 | undefined;
    genTitle(userMessage: CoreUserMessage$1 | undefined): Promise<string>;
    saveMemory({ threadId, memoryConfig, resourceId, userMessages, runId, }: {
        resourceId: string;
        threadId?: string;
        memoryConfig?: MemoryConfig;
        userMessages: CoreMessage$1[];
        time?: Date;
        keyword?: string;
        runId?: string;
    }): Promise<{
        threadId: string;
        messages: any[];
    }>;
    saveResponse({ result, threadId, resourceId, runId, memoryConfig, }: {
        runId: string;
        resourceId: string;
        result: Record<string, any>;
        threadId: string;
        memoryConfig: MemoryConfig | undefined;
    }): Promise<void>;
    sanitizeResponseMessages(messages: Array<CoreToolMessage$1 | CoreAssistantMessage$1>): Array<CoreToolMessage$1 | CoreAssistantMessage$1>;
    convertTools({ toolsets, runId, }: {
        toolsets?: ToolsetsInput;
        threadId?: string;
        runId?: string;
    }): Record<string, CoreTool>;
    preExecute({ resourceId, runId, threadId, memoryConfig, messages, }: {
        runId?: string;
        threadId?: string;
        memoryConfig?: MemoryConfig;
        messages: CoreMessage$1[];
        resourceId: string;
    }): Promise<{
        coreMessages: CoreMessage$1[];
        threadIdToUse: string;
    }>;
    __primitive({ messages, context, threadId, memoryConfig, resourceId, runId, toolsets, }: {
        toolsets?: ToolsetsInput;
        resourceId?: string;
        threadId?: string;
        memoryConfig?: MemoryConfig;
        context?: CoreMessage$1[];
        runId?: string;
        messages: CoreMessage$1[];
    }): {
        before: () => Promise<{
            messageObjects: CoreMessage$1[];
            convertedTools: Record<string, CoreTool> | undefined;
            threadId: string;
        }>;
        after: ({ result, threadId, memoryConfig, outputText, runId, }: {
            runId: string;
            result: Record<string, any>;
            threadId: string;
            memoryConfig: MemoryConfig | undefined;
            outputText: string;
        }) => Promise<void>;
    };
    generate<Z extends ZodSchema | JSONSchema7 | undefined = undefined>(messages: string | string[] | CoreMessage$1[], { context, threadId: threadIdInFn, memoryOptions, resourceId, maxSteps, onStepFinish, runId, toolsets, output, temperature, }?: AgentGenerateOptions<Z>): Promise<GenerateReturn<Z>>;
    stream<Z extends ZodSchema | JSONSchema7 | undefined = undefined>(messages: string | string[] | CoreMessage$1[], { context, threadId: threadIdInFn, memoryOptions, resourceId, maxSteps, onFinish, onStepFinish, runId, toolsets, output, temperature, }?: AgentStreamOptions<Z>): Promise<StreamReturn<Z>>;
}

export { type LMStudioModel as $, Agent as A, type CoreUserMessage as B, type CoreTool as C, type CoreToolMessage as D, type EmbedResult as E, type EmbedManyResult as F, type GenerateReturn as G, type OpenAIConfig as H, type IAction as I, type GoogleModel as J, type GoogleConfig as K, LLM as L, MastraStorage as M, type GoogleGenerativeAISettings as N, type OpenAIModel as O, type AnthropicModel as P, type AnthropicConfig as Q, type RetryConfig as R, type StreamReturn as S, type ToolsInput as T, type GroqModel as U, type GroqConfig as V, type WorkflowOptions as W, type PerplexityModel as X, type PerplexityConfig as Y, type TogetherAiModel as Z, type TogetherAiConfig as _, MastraMemory as a, type LMStudioConfig as a0, type BasetenModel as a1, type BaseTenConfig as a2, type FireworksModel as a3, type FireworksConfig as a4, type MistralModel as a5, type MistralConfig as a6, type XGrokModel as a7, type XGrokConfig as a8, type CustomModelConfig as a9, type StepDef as aA, type StepCondition as aB, type WorkflowContext as aC, type WorkflowLogMessage as aD, type WorkflowEvent as aE, type ResolverFunctionInput as aF, type ResolverFunctionOutput as aG, type SubscriberFunctionOutput as aH, type DependencyCheckOutput as aI, type WorkflowActors as aJ, type WorkflowActionParams as aK, type WorkflowActions as aL, type WorkflowState as aM, type StepId as aN, type ExtractSchemaFromStep as aO, type ExtractStepResult as aP, type StepInputType as aQ, type ExtractSchemaType as aR, type PathsToStringProps as aS, type IExecutionContext as aT, type CohereModel as aa, type CohereConfig as ab, type AzureModel as ac, type AzureConfig as ad, type DeepseekModel as ae, type DeepseekConfig as af, type AmazonModel as ag, type AmazonConfig as ah, type AnthropicVertexModel as ai, type AnthropicVertexConfig as aj, type LLMProvider as ak, type BaseStructuredOutputType as al, type StructuredOutputType as am, type StructuredOutputArrayItem as an, type StructuredOutput as ao, type OutputType as ap, type AiMessageType as aq, type MessageType as ar, type StorageThreadType as as, type MessageResponse as at, type MemoryConfig as au, type SharedMemoryConfig as av, type StepNode as aw, type VariableReference as ax, type BaseCondition as ay, type ActionContext as az, type ModelConfig as b, MastraLLMBase as c, type MastraPrimitives as d, type LLMTextOptions as e, type LLMTextObjectOptions as f, type LLMInnerStreamOptions as g, type LLMStreamObjectOptions as h, type LLMStreamOptions as i, type StepExecutionContext as j, type StepAction as k, type StepVariableType as l, type StepConfig as m, type StepResult as n, type WorkflowRunState as o, type StepGraph as p, type ToolAction as q, type AgentConfig as r, type ToolExecutionContext as s, type TABLE_NAMES as t, type StorageColumn as u, type WorkflowRow as v, type StorageGetMessagesArg as w, type CoreMessage as x, type CoreSystemMessage as y, type CoreAssistantMessage as z };
