import * as ai from 'ai';
import { LanguageModel, Tool, CoreMessage } from 'ai';
import { JSONSchema7 } from 'json-schema';
import { ZodSchema } from 'zod';
import { c as MastraLLMBase, d as MastraPrimitives, T as ToolsInput, e as LLMTextOptions, C as CoreTool, f as LLMTextObjectOptions, g as LLMInnerStreamOptions, h as LLMStreamObjectOptions, i as LLMStreamOptions, G as GenerateReturn, S as StreamReturn } from './index-D-mqcYwN.js';
import './telemetry-oCUM52DG.js';

declare class MastraLLM extends MastraLLMBase {
    #private;
    constructor({ model, mastra }: {
        model: LanguageModel;
        mastra?: MastraPrimitives;
    });
    convertTools(tools?: ToolsInput): Record<string, Tool>;
    __text({ runId, messages, maxSteps, tools, convertedTools, temperature, toolChoice, onStepFinish, }: LLMTextOptions): Promise<ai.GenerateTextResult<{
        [x: string]: CoreTool;
    } | {
        [x: string]: Tool;
    }, never>>;
    __textObject<T>({ messages, onStepFinish, maxSteps, tools, convertedTools, structuredOutput, runId, temperature, }: LLMTextObjectOptions<T>): Promise<ai.GenerateObjectResult<T>>;
    __stream({ messages, onStepFinish, onFinish, maxSteps, tools, convertedTools, runId, temperature, }: LLMInnerStreamOptions): Promise<ai.StreamTextResult<{
        [x: string]: CoreTool;
    } | {
        [x: string]: Tool;
    }, never>>;
    __streamObject<T>({ messages, onStepFinish, onFinish, maxSteps, tools, convertedTools, structuredOutput, runId, temperature, }: LLMStreamObjectOptions<T>): Promise<ai.StreamObjectResult<ai.DeepPartial<T>, T, never>>;
    generate<Z extends ZodSchema | JSONSchema7 | undefined = undefined>(messages: string | string[] | CoreMessage[], { maxSteps, onStepFinish, tools, convertedTools, runId, output, temperature, }?: LLMStreamOptions<Z>): Promise<GenerateReturn<Z>>;
    stream<Z extends ZodSchema | JSONSchema7 | undefined = undefined>(messages: string | string[] | CoreMessage[], { maxSteps, onFinish, onStepFinish, tools, convertedTools, runId, output, temperature, }?: LLMStreamOptions<Z>): Promise<StreamReturn<Z>>;
}

export { MastraLLM as M };
