import { MastraBase } from '../base.js';
import { E as EmbeddingOptions } from '../types-M16hSruO.js';
import '@opentelemetry/api';
import '../index-Cktbxvxx.js';
import 'pino';
import 'stream';
import '../telemetry-oCUM52DG.js';
import '@opentelemetry/sdk-node';
import '@opentelemetry/sdk-trace-base';

interface QueryResult {
    id: string;
    score: number;
    metadata?: Record<string, any>;
    vector?: number[];
}
interface IndexStats {
    dimension: number;
    count: number;
    metric?: 'cosine' | 'euclidean' | 'dotproduct';
}
declare abstract class MastraVector extends MastraBase {
    constructor();
    /**
     * Embeds text using the specified embedding model and options. Always returns embeddings as an array for simplicity
     */
    embed(text: string | string[], options: EmbeddingOptions): Promise<{
        embeddings: number[][];
    }>;
    abstract upsert(indexName: string, vectors: number[][], metadata?: Record<string, any>[], ids?: string[]): Promise<string[]>;
    abstract createIndex(indexName: string, dimension: number, metric?: 'cosine' | 'euclidean' | 'dotproduct'): Promise<void>;
    abstract query(indexName: string, queryVector: number[], topK?: number, filter?: Record<string, any>, includeVector?: boolean): Promise<QueryResult[]>;
    abstract listIndexes(): Promise<string[]>;
    abstract describeIndex(indexName: string): Promise<IndexStats>;
    abstract deleteIndex(indexName: string): Promise<void>;
}

export { type IndexStats, MastraVector, type QueryResult };
